#include "HIVE_JniHelper.h"
#include <android/log.h>
#include <pthread.h>

static pthread_key_t g_key;

/*
extern "C"
{
jint JNI_OnLoad(JavaVM *vm,void *reserved)
{
    return hive::HiveJniHelper::setJavaVM(vm);
}
}
*/

void _hive_detachCurrentThread(void* a) {
    hive::HiveJniHelper::getJavaVM()->DetachCurrentThread();
}

NS_HIVE_BEGIN

#define  TAG "HIVE_SDK_Plugin"
#define  LOGD(...)  __android_log_print(ANDROID_LOG_DEBUG,TAG,__VA_ARGS__)
#define  LOGE(...)  __android_log_print(ANDROID_LOG_ERROR,TAG,__VA_ARGS__)

JavaVM* HiveJniHelper::s_JavaVM = nullptr;
const jint HiveJniHelper::JNI_VERSION = JNI_VERSION_1_4;

jint HiveJniHelper::setJavaVM(JavaVM* vm)
{
    pthread_t thisthread = pthread_self();
    LOGD("HiveJniHelper::setJavaVM(%p), pthread_self() = %ld", vm, thisthread);
    s_JavaVM = vm;

    pthread_key_create(&g_key, _hive_detachCurrentThread);

    return JNI_VERSION;
}

JavaVM* HiveJniHelper::getJavaVM()
{
    return s_JavaVM;
}

JNIEnv* HiveJniHelper::getEnv() {
    JNIEnv *_env = (JNIEnv *)pthread_getspecific(g_key);
    if (_env == nullptr)
        _env = HiveJniHelper::cacheEnv(s_JavaVM);
    return _env;
}


JNIEnv* HiveJniHelper::cacheEnv(JavaVM* jvm) {
    JNIEnv *_env = nullptr;
    // get jni environment
    jint ret = jvm->GetEnv((void**)&_env, JNI_VERSION);
    
    switch (ret) {
    case JNI_OK :
        // Success!
        pthread_setspecific(g_key, _env);
        return _env;
            
    case JNI_EDETACHED :
        // Thread not attached
        if (jvm->AttachCurrentThread(&_env, nullptr) < 0)
            {
                LOGE("Failed to get the environment using AttachCurrentThread()");

                return nullptr;
            } else {
            // Success : Attached and obtained JNIEnv!
            pthread_setspecific(g_key, _env);
            return _env;
        }
            
    case JNI_EVERSION :
        // Cannot recover from this error
        LOGE("JNI interface version 1.4 not supported");
    default :
        LOGE("Failed to get the environment using GetEnv()");
        return nullptr;
    }
}

NS_HIVE_END