#include "HIVE_CppPlugin.h"

NS_HIVE_BEGIN

static ProviderApple::onReportLeaderboard g_ProviderAppleReportLeaderboardListener;
static ProviderApple::onShowLeaderboard g_ProviderAppleShowLeaderboardListener;
static ProviderApple::onLoadAchievements g_ProviderAppleLoadAchievementsListener;
static ProviderApple::onReportAchievement g_ProviderAppleReportAchievementListener;
static ProviderApple::onShowAchievement g_ProviderAppleShowAchievementListener;
static ProviderApple::onResetAchievements g_ProviderAppleResetAchievementsListener;

void ProviderApple::reportScore(std::string score,std::string leaderboardIdentifier,onReportLeaderboard listener) {
    g_ProviderAppleReportLeaderboardListener = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("ProviderApple", "reportScore");
    jsonParam["score"] = picojson::value(score);
    jsonParam["leaderboardIdentifier"] = picojson::value(leaderboardIdentifier);
    HiveCppPlugin::callNative(jsonParam);
}

void ProviderApple::showLeaderboard(onShowLeaderboard listener) {
    g_ProviderAppleShowLeaderboardListener = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("ProviderApple", "showLeaderboard");
    HiveCppPlugin::callNative(jsonParam);
}

void ProviderApple::loadAchievements(onLoadAchievements listener) {
    g_ProviderAppleLoadAchievementsListener = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("ProviderApple", "loadAchievements");
    HiveCppPlugin::callNative(jsonParam);
}

void ProviderApple::reportAchievement(std::string percent,bool showsCompletionBanner,std::string achievementIdentifier,onReportAchievement listener) {
    g_ProviderAppleReportAchievementListener = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("ProviderApple", "reportAchievement");
    jsonParam["percent"] = picojson::value(percent);
    jsonParam["showsCompletionBanner"] = picojson::value(showsCompletionBanner);
    jsonParam["achievementIdentifier"] = picojson::value(achievementIdentifier);
    HiveCppPlugin::callNative(jsonParam);
}

void ProviderApple::showAchievements(onShowAchievement listener) {
    g_ProviderAppleShowAchievementListener = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("ProviderApple", "showAchievements");
    HiveCppPlugin::callNative(jsonParam);
}

void ProviderApple::resetAchievements(onResetAchievements listener) {
    g_ProviderAppleResetAchievementsListener = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("ProviderApple", "resetAchievements");
    HiveCppPlugin::callNative(jsonParam);
}

void ProviderApple::executeEngine(picojson::value jsonParam) {
    ResultAPI result(jsonParam.get("resultAPI"));
    
	std::string methodName = jsonParam.get("method").to_str();
	if(methodName.compare("reportScore") == 0)
    {
        if(g_ProviderAppleReportLeaderboardListener != NULL)
        {
            g_ProviderAppleReportLeaderboardListener(result);
        }
    }
    else if(methodName.compare("showLeaderboard") == 0)
    {
        if(g_ProviderAppleShowLeaderboardListener != NULL)
        {
            g_ProviderAppleShowLeaderboardListener(result);
        }
    }
    else if(methodName.compare("loadAchievements") == 0)
    {
        if(g_ProviderAppleLoadAchievementsListener != NULL)
        {
            std::vector<ProviderAppleAchievement> list;
            if(jsonParam.get("achievementList").is<picojson::null>() == false){
                picojson::array achievementList = jsonParam.get("achievementList").get<picojson::array>();
                for(auto it = achievementList.begin();it != achievementList.end();++it){
                    ProviderAppleAchievement achiev;
                    if((*it).get("identifier").is<picojson::null>() == false)
                        achiev.identifier = (*it).get("identifier").to_str();

                    if((*it).get("percent").is<picojson::null>() == false)
                        achiev.percent = (*it).get("percent").to_str();

                    if((*it).get("completed").is<picojson::null>() == false)
                        achiev.completed = (*it).get("completed").get<bool>();
                    list.push_back(achiev);
                }
            }
            g_ProviderAppleLoadAchievementsListener(result,list);
        }
    }
    else if(methodName.compare("reportAchievement") == 0)
    {
        if(g_ProviderAppleReportAchievementListener != NULL)
        {
            g_ProviderAppleReportAchievementListener(result);
        }
    }
    else if(methodName.compare("showAchievements") == 0)
    {
        if(g_ProviderAppleShowAchievementListener != NULL)
        {
            g_ProviderAppleShowAchievementListener(result);
        }
    }
    else if(methodName.compare("resetAchievements") == 0)
    {
        if(g_ProviderAppleResetAchievementsListener != NULL)
        {
            g_ProviderAppleResetAchievementsListener(result);
        }
    }
}

ProviderAppleAchievement::ProviderAppleAchievement()
:identifier("")
,percent("")
,completed(false) {
}

ProviderAppleAchievement::~ProviderAppleAchievement() {
}

NS_HIVE_END