


#include "HIVE_CppPlugin.h"


NS_HIVE_BEGIN


static SocialFacebook::onProfileFacebook g_onProfileFacebook = NULL;

static SocialFacebook::onProfileFacebook g_onFriendsFacebook = NULL;

static SocialFacebook::onSendMessageFacebook g_onSendMessageFacebook = NULL;

static SocialFacebook::onShowInvitationDialogFacebook g_onShowInvitationDialogFacebook = NULL;

static SocialFacebook::onPostFacebook g_onPostFacebookWithContentURL = NULL;



void SocialFacebook::getMyProfile(onProfileFacebook listener)
{
	g_onProfileFacebook = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("SocialFacebook", "getMyProfile");

	HiveCppPlugin::callNative(jsonParam);
}


void SocialFacebook::getFriends(onProfileFacebook listener)
{
	g_onFriendsFacebook = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("SocialFacebook", "getFriends");

	HiveCppPlugin::callNative(jsonParam);
}


void SocialFacebook::sendMessageFacebook(FacebookMessage messageContents, onSendMessageFacebook listener)
{
	g_onSendMessageFacebook = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("SocialFacebook", "sendMessageFacebook");
    
    picojson::object messageContentsJson = messageContents.toJson();
	jsonParam["messageContents"] = picojson::value(messageContentsJson);

	HiveCppPlugin::callNative(jsonParam);
}


void SocialFacebook::showInvitationDialog(FacebookMessage inviteContents, onShowInvitationDialogFacebook listener)
{
	g_onShowInvitationDialogFacebook = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("SocialFacebook", "showInvitationDialog");

    picojson::object inviteContentsJson = inviteContents.toJson();
    jsonParam["inviteContents"] = picojson::value(inviteContentsJson);

	HiveCppPlugin::callNative(jsonParam);
}


void SocialFacebook::postFacebookWithContentURL(std::string contentURL, onPostFacebook listener)
{
    g_onPostFacebookWithContentURL = listener;
    
    
    picojson::object jsonParam = HiveCppPlugin::createParam("SocialFacebook", "postFacebookWithContentURL");

    jsonParam["contentURL"] = picojson::value(contentURL);
    
    HiveCppPlugin::callNative(jsonParam);
}


bool SocialFacebook::isLogin()
{
    picojson::object jsonParam = HiveCppPlugin::createParam("SocialFacebook", "isLogin");
    
    picojson::value resJson = HiveCppPlugin::callNative(jsonParam);
    
    bool bIsLogin = false;
    picojson::value jsonValue = resJson.get("isLogin");
    if(jsonValue.is<picojson::null>() == false)
        bIsLogin = jsonValue.evaluate_as_boolean();
    
    return bIsLogin;
}
    
    
void SocialFacebook::executeEngine(picojson::value jsonParam)
{
	ResultAPI result(jsonParam.get("resultAPI"));

	std::string methodName = jsonParam.get("method").to_str();
	if(methodName.compare("getMyProfile") == 0)
	{
		if(g_onProfileFacebook != NULL)
		{
            picojson::value jsonValue;

            std::vector<ProfileFacebook> profileList;

            jsonValue = jsonParam.get("profileList");
            if(jsonValue.is<picojson::null>() || jsonValue.is<picojson::array>() == false) {
            	//profileList.push_back(ProfileFacebook());
            }
            else
            {
            	picojson::array profileListJson = jsonValue.get<picojson::array>();
            	for (picojson::array::iterator iter = profileListJson.begin(); iter != profileListJson.end(); ++iter) {
                
            		ProfileFacebook profileFacebook((*iter));
            		profileList.push_back(profileFacebook);
            	}
            }
            
            g_onProfileFacebook(result, profileList);
		}
	}
	else if(methodName.compare("getFriends") == 0)
	{
		if(g_onFriendsFacebook != NULL)
		{
            picojson::value jsonValue;

            std::vector<ProfileFacebook> profileList;

            jsonValue = jsonParam.get("profileList");
            if(jsonValue.is<picojson::null>() || jsonValue.is<picojson::array>() == false) {
            	profileList.push_back(ProfileFacebook());
            }
            else
            {
            	picojson::array profileListJson = jsonValue.get<picojson::array>();
            	for (picojson::array::iterator iter = profileListJson.begin(); iter != profileListJson.end(); ++iter) {
                
            		ProfileFacebook profileFacebook((*iter));
            		profileList.push_back(profileFacebook);
            	}
            }
            
            g_onFriendsFacebook(result, profileList);
		}
	}
	else if(methodName.compare("sendMessageFacebook") == 0)
	{
		if(g_onSendMessageFacebook != NULL)
		{
            g_onSendMessageFacebook(result);
		}
	}
	else if(methodName.compare("showInvitationDialog") == 0)
	{
		if(g_onShowInvitationDialogFacebook != NULL)
		{
            picojson::value jsonValue;

            std::vector<std::string> invitedUserList;

            jsonValue = jsonParam.get("invitedUserList");
            if(jsonValue.is<picojson::null>() || jsonValue.is<picojson::array>() == false) {

            }
            else
            {
            	picojson::array invitedUserListJson = jsonValue.get<picojson::array>();
            	for (picojson::array::iterator iter = invitedUserListJson.begin(); iter != invitedUserListJson.end(); ++iter) {

            		std::string invitedUser = (*iter).to_str();
            		invitedUserList.push_back(invitedUser);
            	}
            }

            g_onShowInvitationDialogFacebook(result, invitedUserList);
		}
	}
    else if(methodName.compare("postFacebookWithContentURL") == 0)
    {
        if(g_onPostFacebookWithContentURL != NULL)
        {
            g_onPostFacebookWithContentURL(result);
        }
    }
}


// ProfileFacebook class Implements
ProfileFacebook::ProfileFacebook()
{
}


ProfileFacebook::ProfileFacebook(picojson::value jsonParam)
{
    if(jsonParam.is<picojson::null>())
        return;
    
    picojson::value jsonValue;
    
    jsonValue = jsonParam.get("uid");
    if(jsonValue.is<picojson::null>() == false)
        this->uid = jsonValue.to_str();
    
    jsonValue = jsonParam.get("email");
    if(jsonValue.is<picojson::null>() == false)
        this->email = jsonValue.to_str();
    
    jsonValue = jsonParam.get("userName");
    if(jsonValue.is<picojson::null>() == false)
        this->userName = jsonValue.to_str();
    
    jsonValue = jsonParam.get("profileImageUrl");
    if(jsonValue.is<picojson::null>() == false)
        this->profileImageUrl = jsonValue.to_str();

    jsonValue = jsonParam.get("emailSecond");
    if(jsonValue.is<picojson::null>() == false)
        this->emailSecond = jsonValue.to_str();
}


picojson::object ProfileFacebook::toJson() const
{
	picojson::object resJson;
	resJson["uid"] = picojson::value(this->uid);
	resJson["email"] = picojson::value(this->email);
	resJson["userName"] = picojson::value(this->userName);
	resJson["profileImageUrl"] = picojson::value(this->profileImageUrl);
	resJson["emailSecond"] = picojson::value(this->emailSecond);
	return resJson;
}


std::string ProfileFacebook::toString() const
{
	picojson::object resJson = this->toJson();
	std::string resJsonString = picojson::value(resJson).serialize();
	return resJsonString;
}


// FacebookMessage class Implements
FacebookMessage::FacebookMessage()
{
}


FacebookMessage::FacebookMessage(std::vector<std::string> arg_recipients, std::string arg_dialogTitle, std::string arg_message, std::string arg_data)
{
    this->recipients = arg_recipients;
    this->dialogTitle = arg_dialogTitle;
    this->message = arg_message;
    this->data = arg_data;
}


FacebookMessage::FacebookMessage(picojson::value jsonParam)
{
    if(jsonParam.is<picojson::null>())
        return;
    
    picojson::value jsonValue;
    
    this->recipients.clear();
    jsonValue = jsonParam.get("recipients");
    if(jsonValue.is<picojson::null>() == false)
    {
		picojson::array jsonArray = jsonValue.get<picojson::array>();
		for (picojson::array::iterator iter = jsonArray.begin(); iter != jsonArray.end(); ++iter) {

			std::string item = (*iter).to_str();
			this->recipients.push_back(item);
		}
    }
    
    jsonValue = jsonParam.get("dialogTitle");
    if(jsonValue.is<picojson::null>() == false)
        this->dialogTitle = jsonValue.to_str();
    
    jsonValue = jsonParam.get("message");
    if(jsonValue.is<picojson::null>() == false)
        this->message = jsonValue.to_str();
    
    jsonValue = jsonParam.get("data");
    if(jsonValue.is<picojson::null>() == false)
        this->data = jsonValue.to_str();
}


picojson::object FacebookMessage::toJson() const
{
    picojson::object resJson;

    picojson::array jsonArray;
    for(int i = 0; i < this->recipients.size(); i++)
    {
    	jsonArray.push_back(picojson::value(this->recipients[i]));
    }
    resJson["recipients"] = picojson::value(jsonArray);

    resJson["dialogTitle"] = picojson::value(this->dialogTitle);
    resJson["message"] = picojson::value(this->message);
    resJson["data"] = picojson::value(this->data);
    return resJson;
}


std::string FacebookMessage::toString() const
{
    picojson::object resJson = this->toJson();
    std::string resJsonString = picojson::value(resJson).serialize();
    return resJsonString;
}




NS_HIVE_END		// namespace hive



