


#include "HIVE_CppPlugin.h"


NS_HIVE_BEGIN


static SocialHive::onProfileHive g_onGetProfileHive = NULL;

static SocialHive::onProfileHive g_onSetProfileHive = NULL;

static SocialHive::onProfileHive g_onFriendsHive = NULL;

static SocialHive::onProfileHive g_onGetProfilesHive = NULL;

static SocialHive::onSendMessageHive g_onSendMessageHive = NULL;

static SocialHive::onSendMessageHive g_onSendInvitationMessage = NULL;

static SocialHive::onShowHiveDialog g_onShowHiveDialog = NULL;

static SocialHive::onGetBadgeInfo g_onGetBadgeInfo = NULL;



static std::string FriendTypeToString(FriendType friendType)
{
	if(friendType == FriendType::IN_GAME)
		return "IN_GAME";
	else if(friendType == FriendType::OUT_GAME)
		return "OUT_GAME";
	else if(friendType == FriendType::INVITED)
		return "INVITED";
	else
		return "ALL_GAME";
}


static std::string HiveDialogTypeToString(HiveDialogType hiveDialogType)
{
	if(hiveDialogType == HiveDialogType::GAME)
		return "GAME";
    else if(hiveDialogType == HiveDialogType::USER)
        return "USER";
    else if(hiveDialogType == HiveDialogType::CAFE)
        return "CAFE";
    else if(hiveDialogType == HiveDialogType::INQUIRY)
        return "INQUIRY";
    else if(hiveDialogType == HiveDialogType::MESSAGE)
        return "MESSAGE";
	else
		return "HOME";
}


void SocialHive::getMyProfile(onProfileHive listener)
{
	g_onGetProfileHive = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("SocialHive", "getMyProfile");

	HiveCppPlugin::callNative(jsonParam);
}


void SocialHive::setMyProfile(std::string comment, onProfileHive listener)
{
	g_onSetProfileHive = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("SocialHive", "setMyProfile");
	jsonParam["comment"] = picojson::value(comment);

	HiveCppPlugin::callNative(jsonParam);
}


void SocialHive::getFriends(FriendType friendType, onProfileHive listener)
{
	g_onFriendsHive = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("SocialHive", "getFriends");

	std::string friendTypeString = FriendTypeToString(friendType);
	jsonParam["friendType"] = picojson::value(friendTypeString);

	HiveCppPlugin::callNative(jsonParam);
}

void SocialHive::getProfiles(std::vector<std::string> vidList, onProfileHive listener)
{
    g_onGetProfilesHive = listener;
    
    picojson::object jsonParam = HiveCppPlugin::createParam("SocialHive", "getProfiles");
    
    picojson::array jsonArray;
    for (std::vector<std::string>::iterator it = vidList.begin(); it != vidList.end(); ++it) {
        jsonArray.push_back(picojson::value((*it)));
    }
    jsonParam["vidList"] = picojson::value(jsonArray);
    
    HiveCppPlugin::callNative(jsonParam);
}

void SocialHive::sendMessage(MessageContent messageContent, onSendMessageHive listener)
{
	g_onSendMessageHive = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("SocialHive", "sendMessage");
	jsonParam["messageContent"] = picojson::value(messageContent.toJson());

	HiveCppPlugin::callNative(jsonParam);
}


void SocialHive::sendInvitationMessage(MessageContent messageContent, onSendMessageHive listener)
{
	g_onSendInvitationMessage = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("SocialHive", "sendInvitationMessage");
	jsonParam["inviteContents"] = picojson::value(messageContent.toJson());

	HiveCppPlugin::callNative(jsonParam);
}


void SocialHive::showHiveDialog(HiveDialogType hiveDialogType, std::string vid, onShowHiveDialog listener)
{
	g_onShowHiveDialog = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("SocialHive", "showHiveDialog");

	std::string hiveDialogTypeString = HiveDialogTypeToString(hiveDialogType);
	jsonParam["hiveDialogType"] = picojson::value(hiveDialogTypeString);
	jsonParam["vid"] = picojson::value(vid);

	HiveCppPlugin::callNative(jsonParam);
}

void SocialHive::getBadgeInfo(onGetBadgeInfo listener)
{
    g_onGetBadgeInfo = listener;

    picojson::object jsonParam = HiveCppPlugin::createParam("SocialHive", "getBadgeInfo");
 
    HiveCppPlugin::callNative(jsonParam);
}


void SocialHive::executeEngine(picojson::value jsonParam)
{
	ResultAPI result(jsonParam.get("resultAPI"));

	std::string methodName = jsonParam.get("method").to_str();
	if(methodName.compare("getMyProfile") == 0)
	{
		if(g_onGetProfileHive != NULL)
		{
            picojson::value jsonValue;

            std::vector<ProfileHive> profileList;

            jsonValue = jsonParam.get("profileList");
            if(jsonValue.is<picojson::null>() || jsonValue.is<picojson::array>() == false) {
            	profileList.push_back(ProfileHive());
            }
            else
            {
				picojson::array profileListJson = jsonValue.get<picojson::array>();
				for (picojson::array::iterator iter = profileListJson.begin(); iter != profileListJson.end(); ++iter) {

					ProfileHive profileFacebook((*iter));
					profileList.push_back(profileFacebook);
				}
            }
            
            g_onGetProfileHive(result, profileList);
		}

	}
	else if(methodName.compare("setMyProfile") == 0)
	{
		if(g_onSetProfileHive != NULL)
		{
            picojson::value jsonValue;

            std::vector<ProfileHive> profileList;

            jsonValue = jsonParam.get("profileList");
            if(jsonValue.is<picojson::null>() || jsonValue.is<picojson::array>() == false) {
            	profileList.push_back(ProfileHive());
            }
            else
            {
				picojson::array profileListJson = jsonValue.get<picojson::array>();
				for (picojson::array::iterator iter = profileListJson.begin(); iter != profileListJson.end(); ++iter) {

					ProfileHive profileFacebook((*iter));
					profileList.push_back(profileFacebook);
				}
            }
            
            g_onSetProfileHive(result, profileList);
		}
	}
	else if(methodName.compare("getFriends") == 0)
	{
		if(g_onFriendsHive != NULL)
		{
            picojson::value jsonValue;

            std::vector<ProfileHive> profileList;

            jsonValue = jsonParam.get("profileList");
            if(jsonValue.is<picojson::null>() || jsonValue.is<picojson::array>() == false) {
            	profileList.push_back(ProfileHive());
            }
            else
            {
				picojson::array profileListJson = jsonValue.get<picojson::array>();
				for (picojson::array::iterator iter = profileListJson.begin(); iter != profileListJson.end(); ++iter) {

					ProfileHive profileHive((*iter));
					profileList.push_back(profileHive);
				}
            }
            
            g_onFriendsHive(result, profileList);
		}
	}
    else if(methodName.compare("getProfiles") == 0)
    {
        if(g_onGetProfilesHive != NULL)
        {
            picojson::value jsonValue;
            
            std::vector<ProfileHive> profileList;
            
            jsonValue = jsonParam.get("profileList");
            if(jsonValue.is<picojson::null>() || jsonValue.is<picojson::array>() == false) {
                profileList.push_back(ProfileHive());
            }
            else
            {
                picojson::array profileListJson = jsonValue.get<picojson::array>();
                for (picojson::array::iterator iter = profileListJson.begin(); iter != profileListJson.end(); ++iter) {
                    
                    ProfileHive profileHive((*iter));
                    profileList.push_back(profileHive);
                }
            }
            
            g_onGetProfilesHive(result, profileList);
        }
    }
	else if(methodName.compare("sendMessage") == 0)
	{
		if(g_onSendMessageHive != NULL)
		{
            g_onSendMessageHive(result);
		}
	}
	else if(methodName.compare("sendInvitationMessage") == 0)
	{
		if(g_onSendInvitationMessage != NULL)
		{
            g_onSendInvitationMessage(result);
		}
	}
	else if(methodName.compare("showHiveDialog") == 0)
	{
		if(g_onShowHiveDialog != NULL)
		{
            g_onShowHiveDialog(result);
		}
	}
    else if(methodName.compare("getBadgeInfo") == 0)
    {
        if(g_onGetBadgeInfo != NULL)
        {
            picojson::value jsonValue;
            jsonValue = jsonParam.get("socialBadge");
            if( !jsonValue.is<picojson::null>() && jsonValue.is<picojson::object>() ) {
                SocialBadge socialBadge(jsonValue);
                g_onGetBadgeInfo(result, socialBadge);
            }
            else{
                g_onGetBadgeInfo(result, SocialBadge());
            }
        }
    }
}




// ProfileHive class Implements
ProfileHive::ProfileHive()
{
}


ProfileHive::ProfileHive(picojson::value jsonParam)
{
    picojson::value jsonValue;
    
    jsonValue = jsonParam.get("vid");
    if(jsonValue.is<picojson::null>() == false)
        this->vid = jsonValue.to_str();

    jsonValue = jsonParam.get("uid");
    if(jsonValue.is<picojson::null>() == false)
        this->uid = jsonValue.to_str();
    
    jsonValue = jsonParam.get("identifier");
    if(jsonValue.is<picojson::null>() == false)
        this->identifier = jsonValue.to_str();
    
    jsonValue = jsonParam.get("userName");
    if(jsonValue.is<picojson::null>() == false)
        this->userName = jsonValue.to_str();
    
    jsonValue = jsonParam.get("facebookId");
    if(jsonValue.is<picojson::null>() == false)
        this->facebookId = jsonValue.to_str();
    
    jsonValue = jsonParam.get("googleplusId");
    if(jsonValue.is<picojson::null>() == false)
        this->googleplusId = jsonValue.to_str();

    jsonValue = jsonParam.get("profileImageUrl");
    if(jsonValue.is<picojson::null>() == false)
        this->profileImageUrl = jsonValue.to_str();
    
    jsonValue = jsonParam.get("country");
    if(jsonValue.is<picojson::null>() == false)
        this->country = jsonValue.to_str();
    
    jsonValue = jsonParam.get("comment");
    if(jsonValue.is<picojson::null>() == false)
        this->comment = jsonValue.to_str();

    jsonValue = jsonParam.get("cached");
    if(jsonValue.is<picojson::null>() == false)
        this->cached = jsonValue.get<bool>();
    
    jsonValue = jsonParam.get("testAccount");
    if(jsonValue.is<picojson::null>() == false)
        this->testAccount = jsonValue.get<bool>();
    
    jsonValue = jsonParam.get("email");
    if(jsonValue.is<picojson::null>() == false)
        this->email = jsonValue.to_str();
    
    jsonValue = jsonParam.get("birthday");
    if(jsonValue.is<picojson::null>() == false)
        this->birthday = jsonValue.to_str();
    
    jsonValue = jsonParam.get("gender");
    if(jsonValue.is<picojson::null>() == false)
        this->gender = jsonValue.to_str();
    
    jsonValue = jsonParam.get("gameFriend");
    if(jsonValue.is<picojson::null>() == false)
        this->gameFriend = jsonValue.get<bool>();
    
    jsonValue = jsonParam.get("assnet");
    if(jsonValue.is<picojson::null>() == false)
        this->assnet = jsonValue.to_str();
}


picojson::object ProfileHive::toJson() const
{
	picojson::object resJson;
	resJson["vid"] = picojson::value(this->vid);
	resJson["uid"] = picojson::value(this->uid);
    resJson["identifier"] = picojson::value(this->identifier);
	resJson["userName"] = picojson::value(this->userName);
	resJson["facebookId"] = picojson::value(this->facebookId);
	resJson["googleplusId"] = picojson::value(this->googleplusId);
	resJson["profileImageUrl"] = picojson::value(this->profileImageUrl);
	resJson["country"] = picojson::value(this->country);
	resJson["cached"] = picojson::value(this->cached);
	resJson["testAccount"] = picojson::value(this->testAccount);
	resJson["email"] = picojson::value(this->email);
    resJson["birthday"] = picojson::value(this->birthday);
    resJson["gender"] = picojson::value(this->gender);
    resJson["gameFriend"] = picojson::value(this->gameFriend);
    resJson["assnet"] = picojson::value(this->assnet);
    resJson["comment"] = picojson::value(this->comment);
	return resJson;
}


std::string ProfileHive::toString() const
{
	picojson::object resJson = this->toJson();
	std::string resJsonString = picojson::value(resJson).serialize();
	return resJsonString;
}




// MessageContent class Implements
MessageContent::MessageContent()
{
	this->usePush = false;
}


MessageContent::MessageContent(std::string arg_vid, std::string arg_uid, std::string arg_message, std::string arg_imageUrl, std::string arg_thumbnailUrl, bool arg_usePush)
{
	this->vid = arg_vid;
	this->uid = arg_uid;
	this->message = arg_message;
	this->imageUrl = arg_imageUrl;
	this->thumbnailUrl = arg_thumbnailUrl;
	this->usePush = arg_usePush;
}


MessageContent::MessageContent(picojson::value jsonParam)
{
    picojson::value jsonValue;
    
    jsonValue = jsonParam.get("vid");
    if(jsonValue.is<picojson::null>() == false)
        this->vid = jsonValue.to_str();
    
    jsonValue = jsonParam.get("uid");
    if(jsonValue.is<picojson::null>() == false)
        this->uid = jsonValue.to_str();
    
    jsonValue = jsonParam.get("message");
    if(jsonValue.is<picojson::null>() == false)
        this->message = jsonValue.to_str();
    
    jsonValue = jsonParam.get("imageUrl");
    if(jsonValue.is<picojson::null>() == false)
        this->imageUrl = jsonValue.to_str();
    
    jsonValue = jsonParam.get("thumbnailUrl");
    if(jsonValue.is<picojson::null>() == false)
        this->thumbnailUrl = jsonValue.to_str();
    
    jsonValue = jsonParam.get("usePush");
    if(jsonValue.is<picojson::null>() == false)
        this->usePush = jsonValue.get<bool>();
}


picojson::object MessageContent::toJson() const
{
	picojson::object resJson;
	resJson["vid"] = picojson::value(this->vid);
	resJson["uid"] = picojson::value(this->uid);
	resJson["message"] = picojson::value(this->message);
	resJson["imageUrl"] = picojson::value(this->imageUrl);
	resJson["thumbnailUrl"] = picojson::value(this->thumbnailUrl);
	resJson["usePush"] = picojson::value(this->usePush);
	return resJson;
}


std::string MessageContent::toString() const
{
	picojson::object resJson = this->toJson();
	std::string resJsonString = picojson::value(resJson).serialize();
	return resJsonString;
}


// SocialBadge class Implements

SocialBadge::SocialBadge()
{
    messageCount = 0;
}

SocialBadge::SocialBadge(int arg_messageCount)
{
    messageCount = arg_messageCount;
}

SocialBadge::SocialBadge(picojson::value jsonParam)
{
    picojson::value jsonValue;
    
    jsonValue = jsonParam.get("messageCount");
    if(jsonValue.is<picojson::null>() == false)
        this->messageCount = jsonValue.get<double>();
}

picojson::object SocialBadge::toJson()
{
    picojson::object resJson;
    resJson["messageCount"] = picojson::value((double)this->messageCount);
    return resJson;
}

std::string SocialBadge::toString()
{
    picojson::object resJson = this->toJson();
    std::string resJsonString = picojson::value(resJson).serialize();
    return resJsonString;
}


NS_HIVE_END		// namespace hive



