package com.hive.sample;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Application;
import android.content.DialogInterface;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import com.com2us.hivesdk.normal.freefull.google.global.android.common.R;
import com.hive.AuthV4;
import com.hive.Configuration;
import com.hive.IAPV4;
import com.hive.PermissionView;
import com.hive.PlatformHelper;
import com.hive.Promotion;
import com.hive.Promotion.EngagementEventState;
import com.hive.Promotion.EngagementEventType;
import com.hive.Promotion.EngagementListener;
import com.hive.ResultAPI;
import com.hive.base.Android;
import com.hive.impl.ConfigurationImpl;
import com.hive.impl.userengagement.UserEngagementCoupon;
import com.hive.sample.game.GamePostboxV4Dialog;
import com.hive.sample.game.GameUser;
import com.hive.sample.game.Post;
import com.hive.sample.uihelper.ViewPagerBase;

import org.json.JSONException;
import org.json.JSONObject;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;


/**
 * HIVE SDK 의 AuthV4 (인증) 기능 테스트 뷰
 *
 * @author hife
 *
 * Created by hife on 2017. 4. 11..
 */
public class AuthV4TestView extends ViewPagerBase implements View.OnClickListener {


    private AuthV4.PlayerInfo recentCurrentPlayerInfo = new AuthV4.PlayerInfo();
    private AuthV4.PlayerInfo recentConflictPlayerInfo = new AuthV4.PlayerInfo();


    public AuthV4TestView(Activity activity, View view, String title) {
        super(activity, view, title);


        initUi();
    }


    @SuppressWarnings({ "unchecked", "rawtypes" })
    private void initUi() {

        createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear);


        ArrayAdapter spProviderAdapter;

        this.view.findViewById(R.id.btnAuthV4RequestPermissionViewData).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4Setup).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4GetSignInType).setOnClickListener(this);

        this.view.findViewById(R.id.btnAuthV4SignIn).setOnClickListener(this);

        spProviderAdapter = new ArrayAdapter(activity, android.R.layout.simple_spinner_item);
        spProviderAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spProviderAdapter.add("GUEST");
        spProviderAdapter.add("HIVE");
        spProviderAdapter.add("FACEBOOK");
        spProviderAdapter.add("GOOGLE");
        spProviderAdapter.add("QQ");
        spProviderAdapter.add("WEIBO");
        spProviderAdapter.add("VK");
        spProviderAdapter.add("WECHAT");
        spProviderAdapter.add("APPLE");
        spProviderAdapter.add("SIGNIN_APPLE");
        spProviderAdapter.add("LINE");
        spProviderAdapter.add("TWITTER");
        spProviderAdapter.add("AUTO");

        ((Spinner)this.view.findViewById(R.id.spProviderType)).setAdapter(spProviderAdapter);

        this.view.findViewById(R.id.btnAuthV4SignOut).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4Connect).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4Disconnect).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4SelectConflict).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4ResolveConflict).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4GetInfo).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4GetProfile).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4ShowSignIn).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4ShowConflictSelection).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4ShowProfile).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4ShowCafe).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4ShowInquiry).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4ShowTerms).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4ShowAdultConfirm).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4CheckProvider).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4CheckMaintenance).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4CheckBlacklist).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4Reset).setOnClickListener(this);
        this.view.findViewById(R.id.btnUserEngagementSetEngagementListener).setOnClickListener(this);
        this.view.findViewById(R.id.btnUserEngagementSetEngagementReady).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4GetProviderFriendsList).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4GetAgeGateU13).setOnClickListener(this);
        this.view.findViewById(R.id.btnPlatformHelperAuthV4RequestUserPermissions).setOnClickListener(this);


        AuthV4.setProviderChangedListener(authV4CheckProviderListener);
    }


    @Override
    public void onClick(View v) {

        int rid = v.getId();

        switch(rid) {

            case R.id.btnAuthV4RequestPermissionViewData: {
                addLog("\n\n=== HIVE SDK 권한 고지 팝업 데이터 요청 ===\n");
                addLog("AuthV4.requestPermissionViewData(PermissionViewDataListener) Called\n");

                AuthV4.requestPermissionViewData(new PermissionView.PermissionViewDataListener() {
                    @Override
                    public void onPermissionViewData(ResultAPI result, PermissionView.PermissionViewData data) {
                        if (result.isSuccess()) {
                            if (result.code == ResultAPI.Code.AuthV4SkipPermissionView) {
                                addLog(String.format("Can skip permission notice"));
                            }
                            else {
                                addLog(String.format("PermissionViewDataListener.onAuthPermissionViewData() Callback\nresult = %s\n", result.toString()));

                                String popupTitle = data.contents;
                                PermissionView.PermissionViewUnitData[] permissionViewUnitDatas = data.permissions;
                                StringBuilder sb = new StringBuilder();
                                for (PermissionView.PermissionViewUnitData permissionViewUnitData : permissionViewUnitDatas) {
                                    sb.append("Title : ").append(permissionViewUnitData.title).append("\n");
                                    sb.append("Description : ").append(permissionViewUnitData.contents).append("\n");
                                    sb.append("Permission Name : ").append(permissionViewUnitData.nativePermissionName).append("\n\n");
                                }

                                PermissionView.PermissionViewUnitData[] commonDatas = data.commons;
                                for (PermissionView.PermissionViewUnitData commonUnitData : commonDatas) {
                                    sb.append("Title : ").append(commonUnitData.title).append("\n");
                                    sb.append("Description : ").append(commonUnitData.contents).append("\n");
                                    sb.append("Permission Name : ").append(commonUnitData.nativePermissionName).append("\n\n");
                                }

                                AlertDialog.Builder builder = new AlertDialog.Builder(activity);
                                builder.setTitle(popupTitle);
                                builder.setMessage(sb.toString());
                                builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {

                                    }
                                });
                                builder.create().show();
                            }
                        } else {
                            addLog(String.format("PermissionViewDataListener.onAuthPermissionViewData() Callback\nresult = %s\n", result.toString()));
                        }
                    }
                });
            }
            break;

            case R.id.btnAuthV4Setup: {

                addLog("\n\n=== HIVE SDK 초기화 요청 ===\n");
                addLog("AuthV4.setup(authV4SetupListener, engagementListener) Called\n");

                AuthV4.setup(new AuthV4.AuthV4SetupListener() {
                    @Override
                    public void onAuthV4Setup(ResultAPI result, boolean isAutoSignIn, String did, ArrayList<AuthV4.ProviderType> providerTypeList) {

                         addLog(String.format("AuthV4SetupListener.onAuthV4Setup() Callback\nresult = %s\n", result.toString()));
                         addLog(String.format("signInType = %b\n", isAutoSignIn));

                        if (result.needExit()) {
//                            System.exit(0);
                            AlertDialog.Builder builder = new AlertDialog.Builder(activity);
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button");
                            builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    Android.finish();
                                }
                            });
                            builder.create().show();
                            return;
                        }

                         // HIVE SDK 초기화 결과에 따라서 로그인 선택 스피너 UI 변경
                         if(result.isSuccess()) {
                             Spinner spinner = (Spinner) AuthV4TestView.this.view.findViewById(R.id.spProviderType);
                             spinner.setSelection(isAutoSignIn ? AuthV4.ProviderType.AUTO.ordinal() : AuthV4.ProviderType.GUEST.ordinal());
                         }

                        String typeList = "";
                        if (providerTypeList != null) {
                            for (AuthV4.ProviderType type :
                                    providerTypeList) {
                                typeList = typeList + "\n  " + type;
                            }
                        }

                         final String msg = result.toString()
                                 + "\nisAutoSignIn: " + isAutoSignIn
                                 + "\ndid: " + did
                                 + "\ntypeList: " + typeList;

                         showAlertDialog("AuthV4 Setup Result", msg);
                     }
                 });

            }
            break;


            case R.id.btnAuthV4GetSignInType: {

                boolean isAutoSignIn = AuthV4.isAutoSignIn();

                addLog(String.format("\n\n=== 현재 사인-인 형태 조회 (GUEST, AUTO) ===\n" +
                        "AuthV4.getSignInType() Called\nsignInType = %s\n", isAutoSignIn ? "AUTO" : "GUEST" ));
                Spinner spinner = (Spinner) this.view.findViewById(R.id.spProviderType);
                spinner.setSelection(isAutoSignIn ? AuthV4.ProviderType.AUTO.ordinal() : AuthV4.ProviderType.GUEST.ordinal());

                Toast toast = Toast.makeText(Configuration.getContext(), "=== isAutoSignIns: " + isAutoSignIn + " ===", Toast.LENGTH_SHORT);
                toast.show();

                Application testAppli = ((Activity) Configuration.getContext()).getApplication();

                Log.w("TEST", "pkg name : " + testAppli.getPackageName());

            }
            break;


            case R.id.btnAuthV4SignIn: {

                Spinner spinner;

                spinner = (Spinner)this.view.findViewById(R.id.spProviderType);
                String selectedValue = (String) spinner.getAdapter().getItem(spinner.getSelectedItemPosition());

                AuthV4.ProviderType providerType = AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US));

                addLog("\n\n=== 자동으로 Guest 혹은 Auto 사인-인을 요청 ===\n");
                addLog(String.format("AuthV4.signIn(providerType = %s, listener) Called\n", providerType.getValue()));

                AuthV4.signIn(providerType, new AuthV4.AuthV4SignInListener() {
                    @Override
                    public void onAuthV4SignIn(ResultAPI result, AuthV4.PlayerInfo playerInfo) {

                        String msg = "AuthV4SignInListener.onAuthV4SignIn() Callback"
                                + "\n" + result
                                + "\n" + playerInfo;

                        addLog(msg);

                        if (result.needExit()) {
//                            System.exit(0);
                            AlertDialog.Builder builder = new AlertDialog.Builder(activity);
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button");
                            builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    Android.finish();
                                }
                            });
                            builder.create().show();
                            return;
                        }

                        showAlertDialog("AuthV4 SignIn Result", msg);

                        if(result.isSuccess()) {
                            GameUser.getInstance().login(null);
                        }
                    }
                });
            }
            break;


            case R.id.btnAuthV4SignOut: {

                addLog("\n\n=== 사인 아웃 요청 ===\n");
                addLog("AuthV4.signOut(listener) Called\n");

                AuthV4.signOut(new AuthV4.AuthV4SignOutListener() {
                    @Override
                    public void onAuthV4SignOut(ResultAPI result) {

                        addLog(String.format("AuthV4SignOutListener.onAuthV4SignOut() Callback\nresult = %s\n", result.toString()));

                        showAlertDialog("AuthV4 SignOut Result", result.toString());

                        GameUser.getInstance().logout(null);

                    }
                });
            }
            break;


            case R.id.btnAuthV4Connect: {

                Spinner spinner;

                spinner = (Spinner)this.view.findViewById(R.id.spProviderType);
                String selectedValue = (String) spinner.getAdapter().getItem(spinner.getSelectedItemPosition());

                AuthV4.ProviderType providerType = AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US));

                addLog("\n\n=== Provider Connect 요청 ===\n");
                addLog(String.format("AuthV4.connect(providerType = %s, listener) Called\n", providerType.name()));

                AuthV4.connect(providerType, new AuthV4.AuthV4ConnectListener() {
                    @Override
                    public void onAuthV4Connect(ResultAPI result, AuthV4.PlayerInfo conflictPlayer) {

                        String msg = "AuthV4ConnectListener.onAuthV4Connect() Callback"
                                + "\n" + result.toString()
                                + "\nCONFLICT: \n" + conflictPlayer;

                        addLog(msg);

                        if (result.needExit()) {
//                            System.exit(0);
                            AlertDialog.Builder builder = new AlertDialog.Builder(activity);
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button");
                            builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    Android.finish();
                                }
                            });
                            builder.create().show();
                            return;
                        }

                        showAlertDialog("AuthV4 connect Result", msg);

                        if(result.code == ResultAPI.Code.AuthV4ConflictPlayer && conflictPlayer != null) {
                            recentCurrentPlayerInfo = AuthV4.getPlayerInfo();
                            recentConflictPlayerInfo = conflictPlayer;
                        }

                    }
                });
            }
            break;


            case R.id.btnAuthV4Disconnect: {

                Spinner spinner;

                spinner = (Spinner)this.view.findViewById(R.id.spProviderType);
                String selectedValue = (String) spinner.getAdapter().getItem(spinner.getSelectedItemPosition());

                AuthV4.ProviderType providerType = AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US));

                addLog("\n\n=== Provider Disconnect 요청 ===\n");
                addLog(String.format("AuthV4.disconnect(providerType = %s, listener) Called\n", providerType.name()));


                AuthV4.disconnect(providerType, new AuthV4.AuthV4DisconnectListener() {
                    @Override
                    public void onAuthV4Disconnect(ResultAPI result) {

                        String msg = "AuthV4ConnectListener.onAuthV4Disconnect() Callback"
                                + "\n" + result.toString();

                        addLog(msg);

                        if (result.needExit()) {
//                            System.exit(0);
                            AlertDialog.Builder builder = new AlertDialog.Builder(activity);
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button");
                            builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    Android.finish();
                                }
                            });
                            builder.create().show();
                            return;
                        }

                        showAlertDialog("AuthV4 connect Result", msg);
                    }
                });
            }
            break;


            case R.id.btnAuthV4SelectConflict: {

                addLog("\n\n=== AuthV4.connect() 메서드를 호출 후 결과 값이 ResultAPI.CONFLICT_PLAYER 이면 이 메서드를 호출 하여 사용자를 선택을 요청 ===\n");
                addLog("AuthV4.selectConflict(currentPlayerId, listener) Called\n");
                addLog("currentPlayerId = " + recentConflictPlayerInfo.playerId);

                AuthV4.selectConflict(recentConflictPlayerInfo.playerId, new AuthV4.AuthV4SignInListener() {
                    @Override
                    public void onAuthV4SignIn(ResultAPI result, AuthV4.PlayerInfo playerInfo) {

                        String msg = "AuthV4SignInListener.onAuthV4SignIn() Callback"
                                + "\n" + result.toString()
                                + "\n" + playerInfo;

                        addLog(msg);

                        if (result.needExit()) {
//                            System.exit(0);
                            AlertDialog.Builder builder = new AlertDialog.Builder(activity);
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button");
                            builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    Android.finish();
                                }
                            });
                            builder.create().show();
                            return;
                        }

                        showAlertDialog("AuthV4 Select Conflict Result", msg);

                        if(result.isSuccess()) {
                            GameUser.getInstance().login(null);
                        }
                    }
                });

            }
            break;


            case R.id.btnAuthV4ResolveConflict: {

                addLog("AuthV4.resolveConflict(listener) Called\n");

                AuthV4.resolveConflict(new AuthV4.AuthV4ResolveConflictListener() {
                    @Override
                    public void onAuthV4ResolveConflict(ResultAPI result) {

                        String msg = "AuthV4ResolveConflictListener.onAuthV4ResolveConflict() Callback"
                                + "\n" + result.toString();

                        addLog(msg);

                        showAlertDialog("AuthV4 Resolve Conflict Result", msg);
                    }
                });

            }
            break;


            case R.id.btnAuthV4GetInfo: {

                addLog("\n\n=== HIVE 인증된 사용자의 정보 조회 ===\n");

                AuthV4.PlayerInfo playerInfo = AuthV4.getPlayerInfo();

                addLog( String.format("AuthV4.getPlayerInfo() Called\n") );

                String msg = "AuthV4ConnectListener.onAuthV4Connect() Callback"
                        + "\n" + playerInfo;

                addLog(msg);

                showAlertDialog("AuthV4 Get Info Result", msg);
            }
            break;


            case R.id.btnAuthV4GetProfile: {

                addLog("\n\n=== 사용자의 프로필 정보 조회 ===\n");

                AuthV4.PlayerInfo playerInfo = AuthV4.getPlayerInfo();

                if (playerInfo == null) {
                    String msg = "Please Sign-In First.";

                    addLog(msg);

                    showAlertDialog("AuthV4 Get Profile Result", msg);
                    break;
                }

                addLog( String.format("AuthV4.getProfile() Called\n") );

                final ArrayList<Long> playerIdList = new ArrayList<Long>();
                playerIdList.add(playerInfo.playerId);

                AuthV4.getProfile(playerIdList, new AuthV4.AuthV4GetProfileListener() {
                    @Override
                    public void onAuthV4GetProfile(ResultAPI result, ArrayList<AuthV4.ProfileInfo> profileInfoList) {

                        AuthV4.ProfileInfo info = null;
                        if (profileInfoList != null) {
                            info = profileInfoList.get(0);
                        }


                        String msg = "AuthV4GetProfileListener.onAuthV4GetProfile() Callback"
                                + "\n" + result + "\n\n" + info;

                        addLog(msg);

                        showAlertDialog("AuthV4 Get Profile Result", msg);
                    }
                });
            }
            break;


            case R.id.btnAuthV4ShowSignIn: {

                addLog("\n\n=== HIVE 사인-인 UI 호출 ===\n");
                addLog(String.format("AuthV4.showSignIn(listener) Called\n"));

                AuthV4.showSignIn(new AuthV4.AuthV4SignInListener() {
                    @Override
                    public void onAuthV4SignIn(ResultAPI result, AuthV4.PlayerInfo playerInfo) {

                        String msg = "AuthV4SignInListener.onAuthV4SignIn() Callback"
                                + "\n" + result.toString()
                                + "\n" + playerInfo;

                        addLog(msg);

                        if (result.needExit()) {
//                            System.exit(0);
                            AlertDialog.Builder builder = new AlertDialog.Builder(activity);
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button");
                            builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    Android.finish();
                                }
                            });
                            builder.create().show();
                            return;
                        }

                        showAlertDialog("AuthV4 Show SignIn Result", msg);

                    }
                });
            }
            break;


            case R.id.btnAuthV4ShowConflictSelection: {

                // 사용자 선택 창, test
                Map<String, Object> currentPlayerData = new HashMap<>();
                currentPlayerData.put("player_id", recentCurrentPlayerInfo.playerId);
                Map<String, Object> currentPlayerGameData = new HashMap<>();
                currentPlayerGameData.put("Name", "CurrentPlayer-" + recentCurrentPlayerInfo.playerId);
                currentPlayerGameData.put("Level", 20);

                currentPlayerData.put("game_data", currentPlayerGameData);


                Map<String, Object> conflictPlayerData = new HashMap<>();
                conflictPlayerData.put("player_id", recentConflictPlayerInfo.playerId);
                Map<String, Object> conflictPlayerGameData = new HashMap<>();
                conflictPlayerGameData.put("Name", "ConflictPlayer-" + recentConflictPlayerInfo.playerId);
                conflictPlayerGameData.put("Level", 10);

                conflictPlayerData.put("game_data", conflictPlayerGameData);


                addLog("\n\n=== AuthV4.connect() 메서드를 호출 후 결과 값이 ResultAPI.CONFLICT_PLAYER 이면 이 메서드를 호출 하여 사용자를 선택을 요청 ===\n");
                addLog("AuthV4.showConflictSelection(currentPlayerData, conflictPlayerData, listener) Called\n");
                addLog(String.format("currentPlayerData = %s\n", currentPlayerData.toString()));
                addLog(String.format("conflictPlayerData = %s\n", conflictPlayerData.toString()));

                AuthV4.showConflictSelection(currentPlayerData, conflictPlayerData, new AuthV4.AuthV4SignInListener() {
                    @Override
                    public void onAuthV4SignIn(ResultAPI result, AuthV4.PlayerInfo playerInfo) {

                        String msg = "AuthV4SignInListener.onAuthV4SignIn() Callback"
                                + "\n" + result.toString()
                                + "\n" + playerInfo;

                        addLog(msg);

                        if (result.needExit()) {
//                            System.exit(0);
                            AlertDialog.Builder builder = new AlertDialog.Builder(activity);
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button");
                            builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    Android.finish();
                                }
                            });
                            builder.create().show();
                            return;
                        }

                        showAlertDialog("AuthV4 Show Conflict Selection Result", msg);

                        if(result.isSuccess()) {
                            GameUser.getInstance().login(null);
                        }

                    }
                });
            }
            break;


            case R.id.btnAuthV4ShowProfile: {

                addLog("\n\n=== 프로필 정보 표시 ===\n");
                addLog(String.format("AuthV4.showProfile(listener) Called\n"));

                if (AuthV4.getPlayerInfo() != null) {

                    AuthV4.showProfile(AuthV4.getPlayerInfo().playerId, new AuthV4.AuthV4ShowProfileListener() {
                        @Override
                        public void onAuthV4ShowProfile(ResultAPI result) {
                            addLog(String.format("AuthV4ShowProfileListener.onAuthV4ShowProfile() Callback\nresult = %s\n", result.toString()));
                            showAlertDialog("AuthV4 Show Profile Result", result.toString());
                        }
                    });

                } else {
                    showAlertDialog("AuthV4 Show Profile Result", "AuthV4 ID is null. Please Sign-In first.");
                }
            }
            break;


            case R.id.btnAuthV4ShowCafe: {

                addLog("\n\n=== HIVE 카페 표시 ===\n");
                addLog(String.format("AuthV4.showCafe(listener) Called\n"));

                AuthV4.showCafe(new AuthV4.AuthV4ShowCafeListener() {
                    @Override
                    public void onAuthV4ShowCafe(ResultAPI result) {
                        addLog(String.format("AuthV4ShowCafeListener.onAuthV4ShowCafe() Callback\nresult = %s\n", result.toString()));
                        showAlertDialog("AuthV4 Show Cafe Result", result.toString());
                    }
                });

            }
            break;


            case R.id.btnAuthV4ShowInquiry: {

                addLog("\n\n=== 1:1 문의 표시 ===\n");
                addLog(String.format("AuthV4.showInquiry(listener) Called\n"));

                AuthV4.showInquiry(new AuthV4.AuthV4ShowInquiryListener() {
                    @Override
                    public void onAuthV4ShowInquiry(ResultAPI result) {
                        addLog(String.format("AuthV4ShowInquiryListener.onAuthV4ShowInquiry() Callback\nresult = %s\n", result.toString()));
                        showAlertDialog("AuthV4 Show Inquiry Result", result.toString());
                    }
                });
            }
            break;


            case R.id.btnAuthV4ShowTerms: {

                addLog("\n\n=== 약관 정보 표시 ===\n");
                addLog(String.format("AuthV4.showTerms(listener) Called\n"));

                AuthV4.showTerms(new AuthV4.AuthV4ShowTermsListener() {
                    @Override
                    public void onAuthV4ShowTerms(ResultAPI result) {
                        addLog(String.format("AuthV4ShowTermsListener.onAuthV4ShowTerms() Callback\nresult = %s\n", result.toString()));
                    }
                });
            }
            break;


            case R.id.btnAuthV4ShowAdultConfirm: {

                addLog("\n\n=== 성인 인증 화면 호출 ===\n");
                addLog("AuthV4.showAdultConfirm(listener) Called\n");

                AuthV4.showAdultConfirm(new AuthV4.AuthV4AdultConfirmListener() {
                    @Override
                    public void onAuthV4AdultConfirm(ResultAPI result) {

                        addLog(String.format("AuthV4AdultConfirmListener.onAuthV4AdultConfirm() Callback\nresult = %s\n", result.toString()));

                        showAlertDialog("AuthV4 Show AdultConfirm Result", result.toString());

                    }
                });
            }
            break;


            case R.id.btnAuthV4CheckProvider: {

                Spinner spinner;

                spinner = (Spinner)this.view.findViewById(R.id.spProviderType);
                String selectedValue = (String) spinner.getAdapter().getItem(spinner.getSelectedItemPosition());

                AuthV4.ProviderType providerType = AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US));

                addLog("\n\n=== Check Provider 요청 ===\n");
                addLog(String.format("AuthV4.checkProvider(providerType = %s, listener) Called\n", providerType.name()));


                AuthV4.checkProvider(providerType, new AuthV4.AuthV4CheckProviderListener() {
                    @Override
                    public void onDeviceProviderInfo(ResultAPI result, AuthV4.ProviderInfo providerInfo) {

                        String msg = "AuthV4SignInListener.onAuthV4SignIn() Callback"
                                + "\n" + result.toString()
                                + "\n" + providerInfo;

                        addLog(msg);

                        showAlertDialog("AuthV4 Check Provider Result", msg);
                    }
                });
            }
            break;


            case R.id.btnAuthV4CheckMaintenance: {

                addLog("\n\n=== 점검 상태 표시 및 데이터 반환 ===\n");
                addLog("AuthV4.checkMaintenance() Called\n");


                AuthV4.checkMaintenance(true, new AuthV4.AuthV4MaintenanceListener() {
                    @Override
                    public void onAuthV4Maintenance(ResultAPI result, ArrayList<AuthV4.AuthV4MaintenanceInfo> maintenanceInfo) {

                        addLog(String.format("AuthV4MaintenanceListener.onAuthV4Maintenance() Callback\nresult = %s\n", result.toString()));

                        if (result.needExit()) {
//                            System.exit(0);
                            AlertDialog.Builder builder = new AlertDialog.Builder(activity);
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button");
                            builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    Android.finish();
                                }
                            });
                            builder.create().show();
                            return;
                        }

                        String msg = new String();

                        if (maintenanceInfo != null && maintenanceInfo.size() > 0) {
                            for (AuthV4.AuthV4MaintenanceInfo authV4MaintenanceInfo :
                                    maintenanceInfo) {

                                msg = msg + authV4MaintenanceInfo.toString() + "\n\n";
                            }

                        } else if (result.isSuccess() && maintenanceInfo.size() <= 0){
                            msg = result.toString() + "\n\n  공지가 없습니다.";
                        }

                        else {
                            msg = result.toString();
                        }

                        showAlertDialog("Auth Check Maintenance Result", msg);
                    }
                });
            }
            break;


            case R.id.btnAuthV4CheckBlacklist: {

                addLog("\n\n=== 제재 상태 표시 및 데이터 반환 ===\n");
                addLog("AuthV4.checkBlacklist() Called\n");

                AuthV4.checkBlacklist(true, new AuthV4.AuthV4MaintenanceListener() {
                    @Override
                    public void onAuthV4Maintenance(ResultAPI result, ArrayList<AuthV4.AuthV4MaintenanceInfo> maintenanceInfo) {

                        addLog(String.format("AuthV4MaintenanceListener.onAuthV4Maintenance() Callback\nresult = %s\n", result.toString()));

                       StringBuffer msg = new StringBuffer();

                        if (result.isSuccess()) {
                            if (maintenanceInfo == null) {
                                // 일반 유저
                                msg.append("Normal User");
                            } else {
                                // 제재 유저
                                msg.append("Blacklist User");
                                msg.append("\n");
                                msg.append(maintenanceInfo.get(0).toString());
                            }
                        }

                        showAlertDialog("AuthV4 Check Blacklist Result", msg.toString());

                    }
                });

            }
            break;


            case R.id.btnAuthV4Reset: {

                addLog("\n\n=== 인증 정보를 포함하여 SDK 에서 사용하는 모든 데이터 초기화 (테스트 용) ===\n");
                addLog("AuthV4.reset() Called\n");


                AuthV4.reset();


                Toast toast = Toast.makeText(Configuration.getContext(), "=== 인증 정보를 포함하여 SDK 에서 사용하는 모든 데이터 초기화 (테스트 용) ===", Toast.LENGTH_SHORT);
                toast.show();


                GameUser.getInstance().logout(null);
            }
            break;

            case R.id.btnUserEngagementSetEngagementListener: {

                addLog("\n\n=== Engagement 리스너 등록 ===\n");
                addLog("UserEngagement.setEngagementListener(listener) Called.\n");

                Promotion.setEngagementListener(this.engagementListener);

                addLog("\n\n=== Engagement 리스너 등록 완료 ===\n");
                addLog("UserEngagement.setEngagementListener(listener) complete.");

                Toast toast = Toast.makeText(Configuration.getContext(),
                        "=== Engagement 리스너 등록 완료 ==="
                        , Toast.LENGTH_SHORT);
                toast.show();
            }
            break;


            case R.id.btnUserEngagementSetEngagementReady: {

                addLog("\n\n=== Engagement 작업 수행이 가능함으로 설정 ===\n");
                addLog("UserEngagement.setEngagementReady() Called.\n");

                ResultAPI retVal = Promotion.setEngagementReady(true);

                addLog("\n\n=== Engagement 작업 수행이 가능함으로 설정 결과 ===\n");
                addLog("UserEngagement.setEngagementReady() result : " + retVal);

                Toast toast = Toast.makeText(Configuration.getContext(),
                        (retVal.isSuccess())
                                ? "=== Engagement 작업 수행이 가능함으로 설정 ==="
                                : "=== Engagement 작업 수행이 가능함으로 설정되지 않음 ==="
                        , Toast.LENGTH_SHORT);
                toast.show();

            }
            break;

            case R.id.btnAuthV4GetProviderFriendsList: {

                Spinner spinner;

                spinner = (Spinner)this.view.findViewById(R.id.spProviderType);
                String selectedValue = (String) spinner.getAdapter().getItem(spinner.getSelectedItemPosition());

                AuthV4.ProviderType providerType = AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US));

                addLog("\n\n=== " + selectedValue + " 친구 조회 결과 ===\n");
                addLog("AuthV4.getProviderFriendsList() Called\n");


                AuthV4.getProviderFriendsList(providerType, new AuthV4.AuthV4ProviderFriendsListener() {
                    @Override
                    public void onGetProviderFriendsList(ResultAPI result, AuthV4.ProviderType providerType, Map<String, Long> providerUserIdList) {
                        addLog(String.format("AuthV4ProviderFriendsListener.onGetProviderFriendsList() Callback\nresult = %s", result.toString()));
                        addLog(String.format("Provider = %s", providerType));
                        if(result.isSuccess() && providerUserIdList != null) {
                            addLog(String.format("friend mathcing ID List = %s\n", providerUserIdList.toString()));
                        }
                    }
                });

            }
            break;

            case R.id.btnAuthV4GetAgeGateU13: {

                addLog("\n\n=== 13세 나이제한 여부 조회 ===\n");
                addLog("AuthV4.getAgeGateU13() Called\n");

                boolean ageGateU13 = AuthV4.getAgeGateU13();

                addLog("AuthV4.getAgeGateU13() result : " + ageGateU13);

                showAlertDialog("AuthV4 getAgeGateU13 Result", "13세 나이제한 여부 : " + ageGateU13);
            }
            break;

            case R.id.btnPlatformHelperAuthV4RequestUserPermissions: {

                EditText editText = this.view.findViewById(R.id.editTextAuthV4PermissionList);
                String[] permissionArray = editText.getText().toString().split("\n");
                List<String> requests = Arrays.asList(permissionArray);

                PlatformHelper.requestUserPermissions( requests, new PlatformHelper.RequestUserPermissionsListener() {
                    @Override
                    public void onRequestUserPermissions(ResultAPI resultApi, List<String> granted, List<String> denied) {

                        addLog("\n\n=== OS 권한동의 팝업 재노출 ===\n");
                        addLog("resultApi :" + resultApi.toString() + "\n");

                        addLog("동의한 권한 목록 : \n");
                        for (String name : granted) {
                            addLog(name + '\n');
                        }
                        addLog("\n거절한 권한 목록: \n");
                        for (String name : denied) {
                            addLog(name + '\n');
                        }
                    }
                });
            }
            break;
        }
    }




    // SDK 가 특정한 조건에서 클라이언트에 개입 (Engagement) 하기 위한 이벤트 리스너
    private EngagementListener engagementListener = new EngagementListener() {

        @Override
        public void onEngagement(ResultAPI result, EngagementEventType engagementEventType, EngagementEventState eventState, JSONObject param) {

            addLog(String.format("AuthInitializeListener.onEngagement() Callback\nresult = %s\n", result.toString()));
            addLog(String.format("engagementEventType = %s , %s\n", engagementEventType.name(), eventState.name()));
            addLog(String.format("param = %s\n", (param == null ? "null" : param.toString())));


            switch (engagementEventType)
            {
                case EVENT_TYPE: {
                    // Engagement의 전체 시작과 끝.
                    if(eventState == EngagementEventState.BEGIN){
                        // Engagement의 모든 동작이 수행됨.
                    } else if(eventState == EngagementEventState.FINISH) {
                        // Engagement의 모든 동작이 수행 완료됨.
                    }
                }
                break;

                case AUTH_LOGIN_VIEW: {

                    if (eventState == EngagementEventState.START) {
                        // HIVE 멤버십 로그인 페이지 노출 요청 시작
                    } else if (eventState == EngagementEventState.END) {
                        // HIVE 멤버십 로그인 수행 완료. (로그인 완료 / 창닫음 수행 이후 전달됨)
                    }
                }
                break;

                case SOCIAL_CAFE_VIEW: {
                    if (eventState == EngagementEventState.START) {
                        // HIVE 카페 페이지 노출 요청 시작
                    } else if (eventState == EngagementEventState.END) {
                        // HIVE 카페 페이지 노출 완료 (HIVE 카페 창 닫은 이후 전달)
                    }
                }
                break;

                case SOCIAL_INQUIRY_VIEW: {
                    if (eventState == EngagementEventState.START) {
                        // 1:1 문의 페이지 노출 요청 시작
                    } else if (eventState == EngagementEventState.END) {
                        // 1:1 문의 페이지 노출 완료 (1:1문의 창 닫은 이후 전달)
                    }
                }
                break;

                case SOCIAL_MESSAGE: {
                    if (eventState == EngagementEventState.START) {
                        // HIVE 메세지 전송 요청 시작
                    } else if (eventState == EngagementEventState.END) {
                        // HIVE 메세지 전송 완료
                    }

                }
                break;

                case SOCIAL_INVITATION_VIEW: {
                    if (eventState == EngagementEventState.START) {
                        // HIVE 초대 메시지 전송 요청 시작
                    } else if (eventState == EngagementEventState.END) {
                        // HIVE 초대 메시지 전송 완료
                    }

                }
                break;

                case PROMOTION_VIEW: {
                    if (eventState == EngagementEventState.START) {
                        // Promotion View 노출 요청 시작
                    } else if (eventState == EngagementEventState.END) {
                        // Promotion View 노출 완료 (Promotion 창 닫은 이후 전달)
                    }

                }
                break;

                case COUPON: {
                    if (eventState == EngagementEventState.START) {
                        // 쿠폰 소모 요청 시작
                    } else if (eventState == EngagementEventState.END) {
                        // 쿠폰 소모 요청 완료 및 응답 전달
                        UserEngagementCoupon.CouponConsumeResult couponConsumeResult = null;
                        String message = null;

                        try {
                            couponConsumeResult = UserEngagementCoupon.CouponConsumeResult.getCouponConsumeResult(param.getInt("code"));
                            message = param.getString("message");
                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                        addLog(String.format("Coupon Code = %s\nCoupon Message = %s\n"
                                , couponConsumeResult.name(), message));
                        Toast toast = Toast.makeText(Configuration.getContext(), "쿠폰 처리 결과입니다.\nCode : " + couponConsumeResult + "\nmMessage : " + message, Toast.LENGTH_SHORT);
                        toast.show();
                    }
                }
                break;

                case OFFERWALL_VIEW: {
                    if (eventState == EngagementEventState.START) {
                        // Offerwall 페이지 노출 요청 시작
                    } else if (eventState == EngagementEventState.END) {
                        // Offerwall 페이지 노출 완료 (Offerwall 창 닫은 이후 전달)
                    }
                }
                break;

                case EVENT: {
                    if (eventState == EngagementEventState.START) {
                        // host가 game인 경우 해당 데이터 전달
                    } else if (eventState == EngagementEventState.END) {
                        // host가 game인 경우 해당 데이터 전달
                    }

                    String api = null;
                    String parameter = null;
                    try {
                        api = param.getString("api");
                        parameter = param.getString("param");
                    }
                    catch (JSONException e) {
                        e.printStackTrace();
                    }

                    addLog(String.format("apiName = %s\napiParam = %s\n", api, parameter));

                    Toast toast = Toast.makeText(Configuration.getContext(), "이벤트를 전달받았습니다.\nAPI : " + api + ", Param : "+ parameter, Toast.LENGTH_SHORT);
                    toast.show();
                }
                break;


                case IAP_UPDATED: {
                    if (eventState == EngagementEventState.START) {
                        // 구매내역이 변경되었으니 받아가라는 ( Restore() 를 호출 ) 신호
                    } else if (eventState == EngagementEventState.END) {
                        // 구매내역이 변경되었으니 받아가라는 ( Restore() 를 호출 ) 신호
                    }
                    Toast toast = Toast.makeText(Configuration.getContext(), "IAP Purchase Updated.", Toast.LENGTH_SHORT);
                    toast.show();
                }
                break;

                //4.5.0 new engagement Callback parameter.
                case IAP_PROMOTE: {
                    if (eventState == EngagementEventState.START) {
                        // IAP Promote 데이터 존재 여부 전달
                    } else if (eventState == EngagementEventState.END) {
                        // IAP Promote 데이터 존재 여부 전달
                    }
                }
                break;

                case IAP_PURCHASE: {
                    if (eventState == EngagementEventState.START) {
                        // IAP Purhcase 구매 시작
                    } else if (eventState == EngagementEventState.END) {
                        // IAP Purhcase 구매 완료 (+영수증데이터)
                        IAPV4.IAPV4Receipt iapV4Receipt = null;

                            try {
                                iapV4Receipt = (IAPV4.IAPV4Receipt) param.get("iapV4Receipt");
                            } catch (JSONException e) {
                                e.printStackTrace();
                            }
                            StringBuilder builder = new StringBuilder();
                            builder.append(String.format("IAPV4PurchaseListener.onIAPV4Purchase() Callback - result = %s\n", result))
                                    .append("IAPV4Receipt : ").append(iapV4Receipt).append("\n");


    //                                                forLog.addLog(builder.toString());
                            addLog(builder.toString());

                            if (result.isSuccess() && iapV4Receipt != null) {

                                GamePostboxV4Dialog.addPost(new Post(
                                        iapV4Receipt.product.marketPid,
                                        iapV4Receipt.product.currency,
                                        (int) iapV4Receipt.product.price,
                                        iapV4Receipt.type.toString(),
                                        iapV4Receipt.additionalInfo,
                                        new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(Calendar.getInstance().getTime())
                                ));

                            }
                            showAlertDialog("Purchase Result", result.toString() + "\n" + builder.toString());
                        }
                    }
                break;
                default:
                    break;

            }
        }
    };


    private AuthV4.AuthV4CheckProviderListener authV4CheckProviderListener = new AuthV4.AuthV4CheckProviderListener() {
        @Override
        public void onDeviceProviderInfo(ResultAPI result, AuthV4.ProviderInfo providerInfo) {

            showAlertDialog("AuthV4 onDeviceProviderInfo Result", result.toString() + "\n" + providerInfo);
        }
    };
}



