package com.hive.sample;


import android.app.Activity;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.com2us.hivesdk.normal.freefull.google.global.android.common.R;
import com.hive.Configuration;
import com.hive.IAP;
import com.hive.IAP.IAPShop;
import com.hive.IAP.IAPType;
import com.hive.ResultAPI;
import com.hive.base.Property;
import com.hive.impl.IAPImpl;
import com.hive.sample.game.GamePostboxDialog;
import com.hive.sample.game.GameUser;
import com.hive.sample.game.IGameListener;
import com.hive.sample.game.Response;
import com.hive.sample.uihelper.ViewPagerBase;

import java.util.List;
import java.util.Locale;




/**
 * HIVE SDK 의 IAP (In App Purchase) 기능 테스트 뷰
 * 
 * @author hife
 */
public class IAPTestView extends ViewPagerBase implements View.OnClickListener {
	
	
	public IAPTestView(Activity activity, View view, String title) {
		super(activity, view, title);

		initUi();
	}


	private void initUi() {

		createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear);
		
		this.view.findViewById(R.id.btnPostbox).setOnClickListener(this);
		this.view.findViewById(R.id.btnIAPInitialize).setOnClickListener(this);
		this.view.findViewById(R.id.btnIAPGetShopInfo).setOnClickListener(this);
		this.view.findViewById(R.id.btnIAPRestore).setOnClickListener(this);
		this.view.findViewById(R.id.btnIAPGetBalance).setOnClickListener(this);
		this.view.findViewById(R.id.btnIAPGetBadge).setOnClickListener(this);
		this.view.findViewById(R.id.btnIAPShowPayment).setOnClickListener(this);
		this.view.findViewById(R.id.btnIAPShowCharge).setOnClickListener(this);

		Property iapProperty = new Property("iap_property");
		String additionalInfo = iapProperty.getValue("iap.additionalInfo");

		if ( !TextUtils.isEmpty(additionalInfo) ) {
			EditText edtAdditionalInfo = (EditText) this.view.findViewById(R.id.edtAdditionalInfo);
			edtAdditionalInfo.setText(additionalInfo);
		}
	}


	public void onGameUserUiUpdate() {
		
		new Handler(Looper.getMainLooper()).post(new Runnable() {
			
			@Override
			public void run() {
				
				TextView tvServerId = (TextView) IAPTestView.this.view.findViewById(R.id.tvIapServerId);
				tvServerId.setText("Server ID: " + Configuration.getServerId());
				
				TextView tvIapNickname = (TextView) IAPTestView.this.view.findViewById(R.id.tvIapNickname);
				tvIapNickname.setText("Name: " + GameUser.getInstance().nickname);
				
				TextView tvIapGold = (TextView) IAPTestView.this.view.findViewById(R.id.tvIapGold);
				tvIapGold.setText(GameUser.getInstance().gold + " GOLD");
				
				TextView tvIapGem = (TextView) IAPTestView.this.view.findViewById(R.id.tvIapGem);
				tvIapGem.setText(GameUser.getInstance().gem + " GEM");
				
			}
		});
	}
	
	
	@Override
	public void onUpdateView() {
		
		onGameUserUiUpdate();
		
		super.onUpdateView();
	}
	
	
	@Override
	public void onClick(View v) {

		int rid = v.getId();
		
		switch(rid) {
		
		
			case R.id.btnPostbox:
			{
				
				if(GameUser.getInstance().isLogin) {
					
					new Handler(Looper.getMainLooper()).post(new Runnable() {
						
						@Override
						public void run() {
							GamePostboxDialog gamePostboxDialog = new GamePostboxDialog(activity, IAPTestView.this);
							gamePostboxDialog.show();
						}
					});
					
				}
				
				else {
					
					GameUser.getInstance().login(new IGameListener() {
						
						@Override
						public void onGameListener(boolean result, Response response) {
							
							if(result && response.isSuccess()) {
								
								new Handler(Looper.getMainLooper()).post(new Runnable() {
									
									@Override
									public void run() {
										
										onGameUserUiUpdate();
										
										GamePostboxDialog gamePostboxDialog = new GamePostboxDialog(activity, IAPTestView.this);
										gamePostboxDialog.show();
									}
								});
							} 
							
							else {
								
								String msg = "Postbox Error : GameUser Login Failed.";
								Toast.makeText(activity, msg, Toast.LENGTH_LONG).show();
							}
							
						}
					});
				}
				
			}
				break;
				
		
			case R.id.btnIAPInitialize:
			{
				addLog("\n\n=== IAP 결재 API 초기화 요청 ===\n");
				addLog("IAP.initialize(listener) Called\n");

				com.hive.IAP.initialize(new com.hive.IAP.IAPMarketInfoListener() {

					@Override
					public void onIAPMarketInfo(ResultAPI result, List<IAPType> iapTypeList) {
						
						addLog(String.format("IAPMarketInfoListener.onIAPMarketInfo() Callback\nresult = %s\niapTypeList = %s\n"
							, result.toString(), iapTypeList != null ? iapTypeList.toString() : "null"));
						
						TextView txtSelectedMarket = (TextView) IAPTestView.this.view.findViewById(R.id.txtSelectedMarket);
						
						if(iapTypeList != null) {
							StringBuffer buffer = new StringBuffer();
							for (IAPType iapType : iapTypeList) {
								buffer.append(" " + iapType.name());
							}
							txtSelectedMarket.setText(buffer.toString());
						}
						
						if(result.isFailure()) {
							showAlertDialog("IAP Initialize Failed", result.toString());
						}
					}

				});
			}
				break;
				
				
			case R.id.btnIAPGetShopInfo:
			{
				// 인자 설정
				EditText edtLocationCode = (EditText) this.view.findViewById(R.id.edtLocationCode);
				String locationCode = edtLocationCode.getText().toString();

				EditText edtAdditionalInfo = (EditText) this.view.findViewById(R.id.edtAdditionalInfo);
				final String additionalInfo = edtAdditionalInfo.getText().toString();

				Property iapProperty = new Property("iap_property");
				iapProperty.setValue("iap.additionalInfo", additionalInfo);
				iapProperty.writeProperties();


				addLog("\n\n=== 상점 정보 조회 ===\n");
				addLog(String.format("IAP.getShopInfo(locationCode = %s, listener) Called\n", locationCode));
				
				com.hive.IAP.getShopInfo(locationCode, new com.hive.IAP.IAPShopInfoListener() {
					
					@Override
					public void onIAPShopInfo(final ResultAPI result, final IAPShop iapShop, final int balance) {

						addLog(String.format("IAPShopInfoListener.onIAPShopInfo() Callback\nresult = %s\niapShop = %s\nbalance = %d\n"
							, result.toString(), iapShop!=null?iapShop.toString():"null", balance));
						
						if(result.isSuccess()) {
							
							TextView txtSelectedMarket = (TextView) IAPTestView.this.view.findViewById(R.id.txtSelectedMarket);

							IAPType selectedIAPType = null;
							for (IAPType iapType : IAPType.values()) {
								if(iapType.getValue() == IAPImpl.getSelectedMarket()) {
									selectedIAPType = iapType;
								}
							}
							txtSelectedMarket.setText(selectedIAPType.name());
							
							new Handler(Looper.getMainLooper()).post(new Runnable() {
								
								@Override
								public void run() {
									IAPShopView iapShopView = new IAPShopView(activity, iapShop, balance, additionalInfo, IAPTestView.this);
									iapShopView.show();
								}
							});
						}
						else {
							
							showAlertDialog("IAP getShopInfo Failed", result.toString());
						}
						
					}
				});
			}
				break;
				
				
			case R.id.btnIAPRestore:
			{
				addLog("\n\n=== HIVE IAP 상품에 대한 미지급된 아이템 지급을 요청. ===\n");
				addLog("IAP.restore(listener) Called\n");

				com.hive.IAP.restoreRecipt(new com.hive.IAP.IAPRestoreReceiptListener() {
					@Override
					public void onIAPRestoreReceipt(final ResultAPI result, IAP.IAPReceipt[] receiptList) {

						StringBuilder builder = new StringBuilder();
						builder.append(String.format("IAPRestoreListener.onIAPRestore() Callback\nresult = %s\n", result.toString()));

						if (receiptList != null) {
							for(IAP.IAPReceipt receipt : receiptList) {

								IAP.IAPProduct product = receipt.product;
								if (product != null)
									builder.append("\n").append(product.toString()).append("\n");
							}
						}

						addLog(builder.toString());
						
						showAlertDialog("IAP Restore result", result.toString() + builder.toString());
					}
				});
			}
				break;
				
				
			case R.id.btnIAPGetBalance:
			{
				addLog("\n\n=== HIVE 러비 상점일 경우 잔액 정보 조회. ===\n");
				addLog("IAP.getBalance(listener) Called\n");
				
				com.hive.IAP.getBalanceInfo(new com.hive.IAP.IAPBalanceInfoListener() {
					
					@Override
					public void onIAPBalance(final ResultAPI result, final int balance) {
						
						addLog(String.format("IAPBalanceListener.onIAPBalance() Callback\nresult = %s\nbalance = %d\n"
							, result.toString(), balance));
						
						showAlertDialog("IAP getBalance result", result.toString() + (result.isSuccess() ? "\n잔액 : " + balance : ""));
					}
				});
			}
				break;

			case R.id.btnIAPShowPayment:
			{
				addLog("\n\n=== 러비 상점이나 구글 상점을 선택하기 위한 창을 띄운다. ===\n");
				addLog("IAP.showPayment(listener) Called\n");
				
				com.hive.IAP.showMarketSelection(new com.hive.IAP.IAPMarketInfoListener() {
					
					@Override
					public void onIAPMarketInfo(final ResultAPI result, final List<IAPType> iapTypeList) {
						
						final String log = String.format("IAPMarketInfoListener.onIAPMarketInfo() Callback\nresult = %s\niapTypeList = %s\n"
								, result.toString(), iapTypeList != null ? iapTypeList.toString() : "null"); 
						
						addLog(log);
						
						TextView txtSelectedMarket = (TextView) IAPTestView.this.view.findViewById(R.id.txtSelectedMarket);
						
						if(iapTypeList != null) {
							StringBuffer buffer = new StringBuffer();
							for (IAPType iapType : iapTypeList) {
								buffer.append(" " + iapType.name());
							}
							
							txtSelectedMarket.setText(buffer.toString());
						}
						
						showAlertDialog("IAP showPayment result", log);
					}
				});
			}
				break;
				
				
			case R.id.btnIAPShowCharge:
			{
		    	addLog("\n\n=== 러비 충전 페이지 호출. ===\n");
		    	addLog("IAP.showCharge(listener) Called\n");
		    	
				com.hive.IAP.showCharge(new com.hive.IAP.IAPBalanceInfoListener() {
					
					@Override
					public void onIAPBalance(ResultAPI result, int balance) {
						
						final String log = String.format(Locale.US, "IAPBalanceListener.onIAPBalance() Callback\nresult = %s\nbalance = %d\n"
								, result.toString(), balance);
						
						addLog(log);
						
						showAlertDialog("IAP ShowCharge Result", log);
					}
				});
			}
				break;
		}

	}
}



