package com.hive.sample;

import android.app.Activity;
import android.view.View;

import com.com2us.hivesdk.normal.freefull.google.global.android.common.R;
import com.hive.ProviderGoogle;
import com.hive.ResultAPI;
import com.hive.sample.uihelper.ViewPagerBase;

/**
 * HIVE SDK 의 AuthV4 (인증) Provider 기능 테스트 뷰
 * <p>
 * Created by hife on 2017. 6. 7..
 */

public class ProviderTestView extends ViewPagerBase implements View.OnClickListener {

    int achievementScore = 0;
    int leaderboardScore = 0;

    public ProviderTestView(Activity activity, View view, String title) {
        super(activity, view, title);

        initUi();
    }

    private void initUi() {

        createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear);

        this.view.findViewById(R.id.btnProviderGoogleAchievementsReveal).setOnClickListener(this);
        this.view.findViewById(R.id.btnProviderGoogleAchievementsUnlock).setOnClickListener(this);
        this.view.findViewById(R.id.btnProviderGoogleAchievementsIncrement).setOnClickListener(this);
        this.view.findViewById(R.id.btnProviderGoogleShowAchievements).setOnClickListener(this);
        this.view.findViewById(R.id.btnProviderGoogleLeaderboardsSubmitScore).setOnClickListener(this);
        this.view.findViewById(R.id.btnProviderGoogleShowLeaderboards).setOnClickListener(this);
        this.view.findViewById(R.id.btnProviderGetGooglePlayerId).setOnClickListener(this);
    }

    @Override
    public void onClick(View v) {

        int rid = v.getId();

        switch (rid) {

            case R.id.btnProviderGoogleAchievementsReveal: {

                addLog("\n\n=== 업적 숨김 해제 요청 ===\n");
                addLog("ProviderGoogle.achievementsReveal() Called\n");

                ProviderGoogle.achievementsReveal("CgkIzev2g9MJEAIQAQ", new ProviderGoogle.GoogleAchievementsListener() {
                    @Override
                    public void onAchievementsResult(ResultAPI resultAPI) {

                        addLog(resultAPI.toString());
                    }
                });
            }
            break;


            case R.id.btnProviderGoogleAchievementsUnlock: {

                addLog("\n\n=== 업적 획득 요청 ===\n");
                addLog("ProviderGoogle.achievementsUnlock() Called\n");

                ProviderGoogle.achievementsUnlock("CgkIzev2g9MJEAIQAw", new ProviderGoogle.GoogleAchievementsListener() {
                    @Override
                    public void onAchievementsResult(ResultAPI resultAPI) {

                        addLog(resultAPI.toString());
                    }
                });
            }
            break;


            case R.id.btnProviderGoogleAchievementsIncrement: {

                addLog("\n\n=== 업적 단계 증가 (+1) / 5 ===\n");
                addLog("ProviderGoogle.achievementsIncrement() Called\n");

                achievementScore++;

                ProviderGoogle.achievementsIncrement("CgkIzev2g9MJEAIQAg", 1);
                ProviderGoogle.achievementsIncrement("CgkIzev2g9MJEAIQBQ", 1);

                ProviderGoogle.achievementsIncrement("CgkIzev2g9MJEAIQBg", 1, new ProviderGoogle.GoogleAchievementsListener() {
                    @Override
                    public void onAchievementsResult(ResultAPI resultAPI) {

                        addLog(resultAPI.toString());
                    }
                });
            }
            break;


            case R.id.btnProviderGoogleShowAchievements: {

                addLog("\n\n=== 업적 UI 표시 ===\n");
                addLog("ProviderGoogle.showAchievements() Called\n");

                ProviderGoogle.showAchievements(new ProviderGoogle.GoogleAchievementsListener() {
                    @Override
                    public void onAchievementsResult(ResultAPI resultAPI) {

                        addLog(resultAPI.toString());
                    }
                });
            }
            break;


            case R.id.btnProviderGoogleLeaderboardsSubmitScore: {

                addLog("\n\n=== 리더보드 점수 증가 (+10) ===\n");
                addLog("ProviderGoogle.achievementsIncrement() Called\n");

                ProviderGoogle.leaderboardsSubmitScore("CgkIzev2g9MJEAIQBw", leaderboardScore += 10, new ProviderGoogle.GoogleLeaderboardsListener() {
                    @Override
                    public void onLeaderboardsResult(ResultAPI resultAPI) {

                        addLog(resultAPI.toString());
                    }
                });
            }
            break;


            case R.id.btnProviderGoogleShowLeaderboards: {

                addLog("\n\n=== 리더보드 UI 표시 ===\n");
                addLog("ProviderGoogle.showLeaderboards() Called\n");

                ProviderGoogle.showLeaderboards(new ProviderGoogle.GoogleLeaderboardsListener() {
                    @Override
                    public void onLeaderboardsResult(ResultAPI resultAPI) {

                        addLog(resultAPI.toString());
                    }
                });
            }
            break;


            case R.id.btnProviderGetGooglePlayerId: {

                addLog("\n\n=== 구글 Player ID 표시 ===\n");
                addLog("ProviderGoogle.getGooglePlayerId() Called\n");

                ProviderGoogle.getGooglePlayerId(new ProviderGoogle.GooglePlayerIdListener() {
                    @Override
                    public void onPlayerIdResult(ResultAPI resultAPI, String googlePlayerId, String authCode) {

                        addLog(resultAPI.toString() + "\ngooglePlayerId : " + googlePlayerId + "\nauthCode : " + authCode);

                    }
                });
            }
            break;
        }

    }
}
