package com.hive.sample.game;


import java.util.ArrayList;

import org.json.JSONArray;

import com.hive.base.Resource;
import com.hive.sample.IAPTestView;
import com.hive.sample.IAPV4TestView;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.BaseAdapter;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;


/**
 * HIVE SDK 의 가상 게임서버와 연결된 우편함을 보여주는 대화 상자
 * 
 * @author hife
 */
public class GamePostboxDialog extends Dialog {

	ListView listview_postbox;
	
	ListViewAdapter listviewadapter_postbox;
	
	IAPTestView parantsView = null;
	IAPV4TestView parantsViewV4 = null;
	

	// IAPv2
	public GamePostboxDialog(final Context context, IAPTestView parantsView) {
		super(context);
		
		this.parantsView = parantsView;

		this.requestWindowFeature(Window.FEATURE_NO_TITLE);
		this.setCancelable(true);
		
		setContentView(Resource.getLayoutId(context, "test_view_iap_postbox"));
		
		listview_postbox = (ListView) findViewById(Resource.getViewId(context, "listview_postbox"));
		
		
		GameNetwork.posts(new IGameListener() {
			
			@Override
			public void onGameListener(boolean result, Response response) {
				
				if(result && response.isSuccess()) {
					
					listviewadapter_postbox = new ListViewAdapter();
					
					listview_postbox.setAdapter(listviewadapter_postbox);
					
					try {
						JSONArray postList = response.resultData.getJSONArray("post_list");

						if(postList.length() == 0 ) {
							TextView textView_postboxtitle = (TextView) findViewById(Resource.getViewId(context, "textView_postboxtitle"));
							textView_postboxtitle.setText("Postbox is empty.");
						}

						for (int i=0; i< postList.length(); i++) {
							listviewadapter_postbox.add( new Post(postList.optJSONObject(i)) );
						}
						
						listview_postbox.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
						
						
					} catch (Exception e) {
						Toast.makeText(context, "Postbox Error : " + e.toString(), Toast.LENGTH_LONG).show();
						dismiss();
					}
					
				} else {
					String msg = "Postbox Error : " + (response == null ? "Invaild Game Session or Network Not Available." : response.resultMsg);
					Toast.makeText(context, msg, Toast.LENGTH_LONG).show();
					dismiss();
				}
				
			}
		});
		
	}

	// IAPv4
	public GamePostboxDialog(final Context context, IAPV4TestView parantsView) {
		super(context);

		this.parantsViewV4 = parantsView;

		this.requestWindowFeature(Window.FEATURE_NO_TITLE);
		this.setCancelable(true);

		setContentView(Resource.getLayoutId(context, "test_view_iap_postbox"));

		listview_postbox = (ListView) findViewById(Resource.getViewId(context, "listview_postbox"));


		GameNetwork.posts(new IGameListener() {

			@Override
			public void onGameListener(boolean result, Response response) {

				if(result && response.isSuccess()) {

					listviewadapter_postbox = new ListViewAdapter();

					listview_postbox.setAdapter(listviewadapter_postbox);

					try {
						JSONArray postList = response.resultData.optJSONArray("post_list");
						JSONArray postSubscriptionList = response.resultData.optJSONArray("post_subscription_list");

						TextView textView_postboxtitle = (TextView) findViewById(Resource.getViewId(context, "textView_postboxtitle"));


						if( postList != null ) {
							for (int i=0; i< postList.length(); i++) {
								listviewadapter_postbox.add( new Post(postList.optJSONObject(i)) );
							}
						}

						if( postSubscriptionList != null ) {
							for (int i=0; i< postSubscriptionList.length(); i++) {
								PostSubscription subscription = new PostSubscription(postSubscriptionList.optJSONObject(i));
								listviewadapter_postbox.add( subscription );
								GameUser.getInstance().subscrptionMarketPid = subscription.getMarketPid();
							}
						}

						if( (postList == null || postList.length() == 0) && (postSubscriptionList == null || postSubscriptionList.length() == 0) ) {

							textView_postboxtitle.setText("Postbox is empty.");
						}

						listview_postbox.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));


					} catch (Exception e) {
						Toast.makeText(context, "Postbox Error : " + e.toString(), Toast.LENGTH_LONG).show();
						dismiss();
					}


				} else {
					String msg = "Postbox Error : " + (response == null ? "Invaild Game Session or Network Not Available." : response.resultMsg);
					Toast.makeText(context, msg, Toast.LENGTH_LONG).show();
					dismiss();
				}

			}
		});

	}


	@Override
	public void dismiss() {
		if (parantsView != null)
			parantsView.onGameUserUiUpdate();

		if (parantsViewV4 != null)
			parantsViewV4.onGameUserUiUpdate();
		super.dismiss();
	}

	
	class ListViewAdapter extends BaseAdapter {

		private ArrayList<PostBoxObject> gamePostList;
		
		public ListViewAdapter() {
			gamePostList = new ArrayList<>();
		}
		
		@Override
		public int getCount() {
			return gamePostList.size();
		}

		@Override
		public Object getItem(int position) {
			return gamePostList.get(position);
		}

		@Override
		public long getItemId(int position) {
			return position;
		}

		@Override
		public View getView(final int position, View convertView, ViewGroup parent) {

			final Context context = parent.getContext();

			final PostBoxObject post = gamePostList.get(position);

			final ViewHolder holder;

			if (convertView == null) {
				
				LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		        convertView = inflater.inflate(Resource.getLayoutId(context, "test_view_iap_post"), parent, false);
		        
		        holder = new ViewHolder();
		        
		        holder.textview_post_assetamount = (TextView) convertView.findViewById(Resource.getViewId(context, "textview_post_assetamount"));
		        holder.textview_post_assetcode = (TextView) convertView.findViewById(Resource.getViewId(context, "textview_post_assetcode"));
				holder.textview_post_character_key = (TextView) convertView.findViewById(Resource.getViewId(context, "textview_post_character_key"));
		        holder.textview_post_usermsg = (TextView) convertView.findViewById(Resource.getViewId(context, "textview_post_usermsg"));
		        holder.textview_post_date_added = (TextView) convertView.findViewById(Resource.getViewId(context, "textview_post_date_added"));

		        holder.gamePost = post;

		        convertView.setTag(holder);
			}
			
			else {
				
				holder = (ViewHolder) convertView.getTag();
			}

			holder.textview_post_assetamount.setText(String.valueOf(post.getAssetAmount()));
			holder.textview_post_assetcode.setText(post.getAssetCode());
			holder.textview_post_character_key.setText(post.getCharaterKey());
			holder.textview_post_usermsg.setText(post.getUserMsg());
			holder.textview_post_date_added.setText(post.getDate());
			
			convertView.setOnClickListener(new View.OnClickListener() {
				
				@Override
				public void onClick(View v) {

					String postId = post.getPostId();
					if( post instanceof PostSubscription )
						return;
					
					if (parantsView != null)
						parantsView.addLog("GameUser Receive Post (postId: " + postId + ")\n");
					if (parantsViewV4 != null)
						parantsViewV4.addLog("GameUser Receive Post (postId: " + postId + ")\n");

					Log.v("sample", "\n\n=== Game 우편 수신 ===\n");
					Log.v("sample", String.format("GameUser Receive Post (postId: " + postId + ")\n", postId));
			        
			        GameNetwork.postReceive(new IGameListener() {
						
						@Override
						public void onGameListener(boolean result, Response response) {
							
							String msg = null;
							
							if(result) {
								
								if(response.isSuccess()) {
									msg = "GameUser Receive Post Success.";
									
									listviewadapter_postbox.remove(position);
								
								} else {
									msg = "GameUser Receive Post Failure. : " + response.resultMsg;
								}
								
							} else {
								msg = "GameUser Receive Post Failure.";
							}
							
							if (parantsView != null)
								parantsView.addLog(msg);

							if (parantsViewV4 != null)
								parantsViewV4.addLog(msg);
							Log.v("sample", msg);
							
							
							new AlertDialog.Builder(context)
							.setTitle("Receive Post Result")
							.setMessage(msg)
							.create().show();
							
//							Toast.makeText(context, msg, Toast.LENGTH_SHORT).show();
							
						}
					}, post);
			        
				}

			});

			
			return convertView;
		}
		
		public void add(PostBoxObject Post) {
			gamePostList.add(Post);
			this.notifyDataSetChanged();
		}
		
		public void remove(int position) {
			gamePostList.remove(position);
			this.notifyDataSetChanged();
		}
	}
	
	
	class ViewHolder {
		
		TextView textview_post_assetamount;
		TextView textview_post_assetcode;
		TextView textview_post_character_key;
		TextView textview_post_usermsg;
		TextView textview_post_date_added;

		PostBoxObject gamePost;
	}
	
}
