package com.hive.sample.game;


import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.hive.IAPV4;
import com.hive.ResultAPI;
import com.hive.base.Android;
import com.hive.base.Property;
import com.hive.base.Resource;
import com.hive.sample.IAPV4TestView;

import java.util.ArrayList;
import java.util.List;


/**
 * HIVE SDK 의 가상 게임서버와 연결된 우편함을 보여주는 대화 상자
 * 
 * @author hife
 */
public class GamePostboxV4Dialog extends Dialog {

	Button btn_postbox_consume;
	ListView listview_postbox;

	ListViewAdapter listviewadapter_postbox;

	IAPV4TestView parantsView = null;


	public GamePostboxV4Dialog(final Context context, final IAPV4TestView parantsView) {
		super(context);
		
		this.parantsView = parantsView;

		this.requestWindowFeature(Window.FEATURE_NO_TITLE);
		this.setCancelable(true);
		
		setContentView(Resource.getLayoutId(context, "test_view_iap_postbox"));

		btn_postbox_consume = (Button) findViewById(Resource.getViewId(context, "textView_postbox_consume"));
		btn_postbox_consume.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {

				if(listviewadapter_postbox.getCount() <= 0) {
					new AlertDialog.Builder(context)
							.setTitle("Multi Receive Post Result")
							.setMessage("Postbox is empty.")
							.create().show();

				} else {
					ArrayList<String> marketPidList = new ArrayList<String>();

					for (PostBoxObject post : listviewadapter_postbox.gamePostList) {
						marketPidList.add(post.getMarketPid());
					}

					IAPV4.transactionMultiFinish(marketPidList, new IAPV4.IAPV4TransactionMultiFinishListener() {
						@Override
						public void onIAPV4TransacionMultiFinish(List<ResultAPI> resultList, List<String> marketPidList) {
							StringBuffer msg = new StringBuffer();

							for (int i = 0; i < resultList.size(); i++) {
								ResultAPI resultAPI = resultList.get(i);
								String marketPid = marketPidList.get(i);

								msg.append("\n" + marketPid + " : ");

								if (resultAPI.isSuccess()) {
									for (PostBoxObject gamePost : listviewadapter_postbox.gamePostList) {
										if (TextUtils.equals(gamePost.getMarketPid(), marketPid)) {

											if (listviewadapter_postbox.remove(gamePost)) {
												msg.append("Receive Success. : plus " + gamePost.getAssetAmount() + " gem");

//												GameUser.getInstance().gem += gamePost.getAssetAmount();
											} else {
												msg.append("Receive Failed. : \n " + resultAPI);
											}
											break;
										}
									}
								}

								msg.append("\n\n");

							}

							if (parantsView != null)
								parantsView.addLog(msg.toString());

							Log.v("sample", msg.toString());

							new AlertDialog.Builder(context)
									.setTitle("Multi Receive Post Result")
									.setMessage(msg.toString())
									.create().show();
						}
					});
				}

			}
		});

		listview_postbox = (ListView) findViewById(Resource.getViewId(context, "listview_postbox"));
		
		listviewadapter_postbox = new ListViewAdapter( loadPost() );

		listview_postbox.setAdapter(listviewadapter_postbox);

		try {
			if(listviewadapter_postbox.getCount() <= 0) {
				TextView textView_postboxtitle = (TextView) findViewById(Resource.getViewId(context, "textView_postboxtitle"));
				textView_postboxtitle.setText("Postbox is empty.");

//			} else {
//				for (Post post : posts) {
//					listviewadapter_postbox.add( post );
//				}
			}

			listview_postbox.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));

		} catch (Exception e) {
			Toast.makeText(context, "Postbox Error : " + e.toString(), Toast.LENGTH_LONG).show();
			dismiss();
		}


	}


	@Override
	public void dismiss() {
		if (parantsView != null)
			parantsView.onGameUserUiUpdate();
		super.dismiss();
	}


	public static void addPost(PostBoxObject post) {

		ArrayList<PostBoxObject> posts = loadPost();

		int index = posts.indexOf(post);

		if (index < 0)
			posts.add(post);
		else
			posts.set(index, post);

		savePost(posts);
	}


	static void savePost(ArrayList<PostBoxObject> posts) {

		String hexStr = Android.serialize(posts);

		if (!TextUtils.isEmpty(hexStr)) {
			Property iapProperty = new Property("iapv4_property");
			iapProperty.loadProperties();
			iapProperty.setValue("iapv4sample.posts", hexStr);
			iapProperty.writeProperties();
		}
	}


	static ArrayList<PostBoxObject> loadPost() {

		Property iapProperty = new Property("iapv4_property");
		iapProperty.loadProperties();
		String hexStr = iapProperty.getValue("iapv4sample.posts");

		if (TextUtils.isEmpty(hexStr)) {
			return new ArrayList<>();

		} else {
			Object obj =  Android.deserialize(hexStr);

			if (obj != null && obj instanceof ArrayList)
				return (ArrayList<PostBoxObject>) obj;
			else return new ArrayList<>();
		}
	}


	class ListViewAdapter extends BaseAdapter {

		private ArrayList<PostBoxObject> gamePostList;
		
		public ListViewAdapter(ArrayList<PostBoxObject> posts) {
			if (posts != null)
				gamePostList = posts;
			else
				gamePostList = new ArrayList<>();
		}
		
		@Override
		public int getCount() {
			return gamePostList.size();
		}

		@Override
		public Object getItem(int position) {
			return gamePostList.get(position);
		}

		@Override
		public long getItemId(int position) {
			return position;
		}

		@Override
		public View getView(final int position, View convertView, ViewGroup parent) {

			final Context context = parent.getContext();
			
			final PostBoxObject post = gamePostList.get(position);

			final ViewHolder holder;

			if (convertView == null) {
				
				LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		        convertView = inflater.inflate(Resource.getLayoutId(context, "test_view_iap_post"), parent, false);
		        
		        holder = new ViewHolder();
		        
		        holder.textview_post_assetamount = (TextView) convertView.findViewById(Resource.getViewId(context, "textview_post_assetamount"));
		        holder.textview_post_assetcode = (TextView) convertView.findViewById(Resource.getViewId(context, "textview_post_assetcode"));
				holder.textview_post_character_key = (TextView) convertView.findViewById(Resource.getViewId(context, "textview_post_character_key"));
		        holder.textview_post_usermsg = (TextView) convertView.findViewById(Resource.getViewId(context, "textview_post_usermsg"));
		        holder.textview_post_date_added = (TextView) convertView.findViewById(Resource.getViewId(context, "textview_post_date_added"));

				holder.gamePost = post;

		        convertView.setTag(holder);
			}
			
			else {
				
				holder = (ViewHolder) convertView.getTag();
			}

			holder.textview_post_assetamount.setText(String.valueOf(post.getAssetAmount()));
			holder.textview_post_assetcode.setText(post.getAssetCode());
			holder.textview_post_character_key.setText(post.getCharaterKey());
			holder.textview_post_usermsg.setText(post.getUserMsg());
			holder.textview_post_date_added.setText(post.getDate());

			
			convertView.setOnClickListener(new View.OnClickListener() {
				int increasedGem = 0;
				@Override
				public void onClick(View v) {

					String postId = post.getMarketPid();
					
					if (parantsView != null)
						parantsView.addLog("GameUser Receive Post (postId: " + postId + ")\n");

					Log.v("sample", "\n\n=== Game 우편 수신 ===\n");
					Log.v("sample", String.format("GameUser Receive Post (postId: " + postId + ")\n", postId));

					if ( listviewadapter_postbox.remove(post) ) {

						IAPV4.transactionFinish(postId, new IAPV4.IAPV4TransactionFinishListener() {
							@Override
							public void onIAPV4TransacionFinish(ResultAPI result, String marketPid) {
								Log.v("sample", "transactionFinish : " + result);

								String msg;

								if (result.isSuccess()) {
									msg = "GameUser Receive Post Success. : plus " + increasedGem + " gem";

									GameUser.getInstance().gem += increasedGem;

								} else {
									msg = "GameUser Receive Post Failed. : \n " + result;
								}

								if (parantsView != null)
									parantsView.addLog(msg);

								Log.v("sample", msg);

								new AlertDialog.Builder(context)
										.setTitle("Receive Post Result")
										.setMessage(msg)
										.create().show();
							}
						});

					} else {

						new AlertDialog.Builder(context)
								.setTitle("Receive Post Result")
								.setMessage("Receive post failed. Post local data error, please try restore().")
								.create().show();
					}

				}

			});

			
			return convertView;
		}


		public void add(PostBoxObject post) {

			int index = gamePostList.indexOf(post);

			if (index < 0)
				gamePostList.add(post);
			else
				gamePostList.set(index, post);

			savePost(gamePostList);
			this.notifyDataSetChanged();
		}


		public boolean remove(PostBoxObject post) {

			if (gamePostList.remove(post) ) {
				savePost(gamePostList);
				this.notifyDataSetChanged();
				return true;

			} else {
				return false;
			}
		}
	}
	
	
	class ViewHolder {
		
		TextView textview_post_assetamount;
		TextView textview_post_assetcode;
		TextView textview_post_character_key;
		TextView textview_post_usermsg;
		TextView textview_post_date_added;

		PostBoxObject gamePost;
	}
	
}
