package com.hive.sample.uihelper;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;


import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Handler;
import android.os.Looper;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.TextView;


import org.json.JSONObject;

import com.hive.base.Logger;




/**
 * 테스터 UI 를 각 기능별로 좌/우로 스크롤 해서 구성하기 위한 Android ViewPager 의 기반 클래스
 * 
 * @author ryuvsken
 */
public class ViewPagerBase {

	
	protected String TAG = com.hive.Const.TAG.concat("_sample");
	
	
	protected Activity activity;
	
	protected View view;
	
	protected String title;
	
	
    // 로그 뷰
    protected LogTextView logView;
    
    
    	

	public ViewPagerBase(Activity activity, View view, String title) {
		this.activity = activity;
		this.view = view;
		this.title = title;
	}
	
	
	public Activity getActivity() {
		return this.activity;
	}
	
	
	public View getView() {
		return this.view;
	}
	
	
	public String getTitle() {
		return this.title;
	}
	
	
    // View Pager Event
	public void onAddLog(String log) {}
	
	public void onUpdateView() {}
	
	public void onBackPressed() {}
	
	public Boolean onActivityResult(int requestCode, int resultCode, Intent data) {return false;}
	
	
	
	
    // 이 클래스를 상속받은 Activity 는 onCreate() 함수에서 반드시 이 메서드를 호출해주어야 한다.
    // 
    // @param id Layout 에 배치된 LogTextView 의 id
    protected void createLogView(final int viewId, final int btnIdLogDialog, final int btnIdLogClear) {
    	
    	final LogTextView logView = (LogTextView)this.view.findViewById(viewId);
    	this.logView = logView;
    	if(logView != null) {
            logView.setMovementMethod(new ScrollingMovementMethod());
            //logView.setOnCreateContextMenuListener(this);
    	}
    	
    	Button button = (Button)this.view.findViewById(btnIdLogDialog);
    	button.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {

				showLogDialog(activity, logView.getText().toString());
			}
		});
    	
    	button = (Button)this.view.findViewById(btnIdLogClear);
    	button.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {

				logView.setText("");
			}
		});
    	
    	//button.requestFocus();
    }
    
    
    private static String divideString = "======";
    
    public void addLogS(final String message) {
    	
    	addLog("");
    	
    	if(message != null) {
    		addLog(divideString + " Start " + message + " " + divideString);
    	}
    	else {
    		addLog(divideString + " Start " + divideString);
    	}
    }
    
    public void addLogE(final String message) {
    	
    	if(message != null) {
    		addLog(divideString + " End " + message + " " + divideString);
    	}
    	else {
    		addLog(divideString + " End " + divideString);
    	}
    	
    	addLog("");
    }
    
    public void addLogD(final String message) {
    	
    	addLog(divideString);
    	addLog(message);
    	addLog(divideString);
    }
    
    
	public void addLog(String message) {
		
		if(message == null) {
			message = "(null)";
		}
		
		//Log.d(TAG, message);
		Log.v("sample", message + "\n");
		
		ViewPagerManager pagerAdapterManager = ViewPagerManager.getInstance();
		List<ViewPagerBase> pageViewList = pagerAdapterManager.getPageViewList();
		
		for(ViewPagerBase view : pageViewList) {
			view.onAddLog(message);
		}

//		Time now = new Time();
//		now.setToNow();
		SimpleDateFormat sdf = new SimpleDateFormat("HHmmss");
		String currentDateandTime = sdf.format(new Date());
		final String finalMessage = currentDateandTime + message;
		if(logView != null) {
			
			activity.runOnUiThread(new Runnable() {
	
				@Override
				public void run() {
					logView.append((String) finalMessage + "\n");
				}
			});
		}
	}
	
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public void addLog(final Map map) {
		
		StringBuffer sb = new StringBuffer();
		
		Set<String> keys = map.keySet();
		for(String key : keys) {
			
			Object obj = (Object) map.get(key);
			if(obj instanceof List) {
				
				List<String> values = (List<String>)obj;
				for(String value : values) {
					sb.append(key).append("=").append(value).append("\r\n");
				}
			}
			else {
				String value = (String) obj;
				sb.append(key).append("=").append(value).append("\r\n");
			}
		}
		
		String text = sb.toString();
		addLog(text);
	}
	
	
	public void addLog(JSONObject json) {
		
		String text = "";
		try {
			text = json.toString(2);
		}
		catch (Exception e) {
			//e.printStackTrace();
			return;
		}
		
		addLog(text);
	}
	
	
	// 샘플 앱의 Alert 대화 상자를 노출 한다.
	public void showAlertDialog(String title, String msg) {
		
		if(title == null) {
			title = "(null)";
		}
		
		if(msg == null) {
			msg = "(null)";
		}
		
		final String fTitle = title;
		final String fMsg = msg;
		
		new Handler(Looper.getMainLooper()).post(new Runnable() {
			
			@Override
			public void run() {
				new AlertDialog.Builder(activity)
				.setTitle(fTitle).setMessage(fMsg).create().show();
			}
		});
	}
	
	
	// 샘플 앱의 로그 대화 상자를 노출 한다.
	public void showLogDialog(int resourceId) {
		
		String message = (String) activity.getString(resourceId);
		ViewPagerBase.showLogDialog(activity, message);
	}
	
	
	public static void showLogDialog(final Activity activity, final String message) {
		
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(activity);

        final View rootView = buildCustomView(activity, message);
        if(rootView == null) {
        	return;
        }
        dialogBuilder.setView(rootView);
        dialogBuilder.setPositiveButton("Clipboard", new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				
		        if (which == DialogInterface.BUTTON_POSITIVE) {
		        	
		        	final TextView txtMessage = (TextView) rootView.findViewById(1);
		        	com.hive.base.Android.copyToClipboard(activity, "logText", txtMessage.getText().toString());
		        	
		        }
		        else {}
			}
        	
        });
        
        dialogBuilder.setNegativeButton(activity.getText(android.R.string.cancel), new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {}
        });
        
        final AlertDialog alertDialog = dialogBuilder.create();
        alertDialog.setCanceledOnTouchOutside(false);
        alertDialog.setOnDismissListener(new DialogInterface.OnDismissListener() {

			@Override
			public void onDismiss(DialogInterface dialog) {}
        });
        alertDialog.show();
	}
	
	protected static View buildCustomView(final Activity activity, final String message) {
		
        final LinearLayout root = new LinearLayout(activity);
        root.setOrientation(LinearLayout.VERTICAL);
        root.setPadding(10, 10, 10, 10);
        root.setLayoutParams(new LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT));
        root.setFocusable(true);
        root.setFocusableInTouchMode(true);

        final ScrollView scroll = new ScrollView(activity);
        root.addView(scroll, new LinearLayout.LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT, 1.0f));
        final LinearLayout scrollable = new LinearLayout(activity);
        scrollable.setOrientation(LinearLayout.VERTICAL);
        scroll.addView(scrollable);

        final TextView text = new TextView(activity);
        text.setId(1);
        text.setText(message);
        scrollable.addView(text);

        return root;
	}
	
	
	public void updateView() {
		
		ViewPagerManager pagerAdapterManager = ViewPagerManager.getInstance();
		List<ViewPagerBase> pageViewList = pagerAdapterManager.getPageViewList();
		
		for(ViewPagerBase view : pageViewList) {
			view.onUpdateView();
		}
	}
}



