package com.hive.sample.uihelper;

import java.util.ArrayList;
import java.util.List;

import com.com2us.hivesdk.normal.freefull.google.global.android.common.R;

import android.view.View;
import android.app.Activity;
import android.os.Parcelable;

import androidx.viewpager.widget.PagerAdapter;
import androidx.viewpager.widget.ViewPager;


/**
 * 테스터 UI 를 각 기능별로 좌/우로 스크롤 해서 구성하기 위한 Android ViewPager 의 관리 클래스
 * 
 * @author ryuvsken
 */
public class ViewPagerManager extends PagerAdapter {

	private static final ViewPagerManager pagerAdapterManager = new ViewPagerManager();

	
	private Activity activity;
	
	private ViewPager viewPager;

	private List<ViewPagerBase> pageViewList = new ArrayList<ViewPagerBase>();
	
	private int currentPosition = 0;
	
	private ViewPagerIndicator viewPagerIndicator;
	
	

	public static ViewPagerManager getInstance() {
		
		return ViewPagerManager.pagerAdapterManager;
	}
	
	
	public Activity getActivity() {
		return this.activity;
	}
	
	public void setActivity(Activity activity) {
		this.activity = activity;
	}
	
	
	public List<ViewPagerBase> getPageViewList() {
		return this.pageViewList;
	}
	
	
	public ViewPager getViewPager() {
		return this.viewPager;
	}
	
	
	public void setViewPager(ViewPager viewPager) {
		
		this.viewPager = viewPager;
		
		
		// 현재 페이지의 위치를 표시하기 위한 인디케이터 초기화
		this.viewPagerIndicator = (ViewPagerIndicator) this.activity.findViewById(R.id.viewPagerIndicator);
		//원사이의 간격
		this.viewPagerIndicator.setItemMargin(15);
        //애니메이션 속도
		this.viewPagerIndicator.setAnimDuration(300);
        //indecator 생성
		this.viewPagerIndicator.createDotPanel(this.pageViewList.size(), 0 , 0);
	}
	
	public void releaseViewPager() {
		
		this.viewPager = null;
		clearViewList();
	}

	
	public void addView(ViewPagerBase v) {
		
		this.pageViewList.add(v);
	}
	
	private void clearViewList() {
		
		this.pageViewList.clear();
	}
	
	
	public ViewPagerBase getViewByPosition(int i) {
		
		return pageViewList.get(i);
	}
	
	
	@Override
	public int getCount() {
		
		return pageViewList.size();
	}
	
	@Override
	public CharSequence getPageTitle(int position) {
		return pageViewList.get(position).getTitle();
	}
	
	
	@Override
	public Object instantiateItem(View pager, int position) {

		View v = pageViewList.get(position).getView();

		((ViewPager) pager).addView(v, 0);

		return v;
	}

	
	@Override
	public void destroyItem(View pager, int position, Object view) {
		((ViewPager) pager).removeView((View) view);
	}

	
	@Override
	public boolean isViewFromObject(View pager, Object obj) {
		return pager == obj;
	}

	
	@Override
	public void restoreState(Parcelable arg0, ClassLoader arg1) {}

	
	@Override
	public Parcelable saveState() {
		return null;
	}
	

	@Override
	public void startUpdate(View arg0) {}

	
	@Override
	public void finishUpdate(View arg0) {}
	
	
	
	@Override
	public int getItemPosition(Object object) {
		return POSITION_NONE;
	}
	
	
	public int getCurrentPosition() {
		return this.currentPosition;
	}

	
	public void setCurrentPosition(int currentPosition) {
		if(currentPosition < 0 || currentPosition >= pageViewList.size()) {
			return;
		}
		
		this.currentPosition = currentPosition;
		this.viewPager.setCurrentItem(currentPosition);
	}
	

	public void onPageChange(int position) {
		this.currentPosition = position;
		
		this.viewPagerIndicator.selectDot(position);
		
		this.pageViewList.get(position).onUpdateView();
	}
}



